from typing import Callable

from PyQt5.QtCore import Qt, QRect, QPoint
from PyQt5.QtWidgets import QSizePolicy

from .flow_layout import FlowLayout


class LibraryLayout(FlowLayout):
    def __init__(self, parent=None):
        super(LibraryLayout, self).__init__(parent)

    def expandingDirections(self) -> Qt.Orientations:
        return Qt.Orientations(Qt.Orientation(0))
        # return Qt.Horizontal | Qt.Vertical

    def setGeometry(self, a0: QRect) -> None:
        super(FlowLayout, self).setGeometry(a0)
        self.doLayout(a0, False)

    def doLayout(self, rect, testonly):
        """!
        @brief Arranges the widgets for this layout

        <pre>
        Layout
        +-----------------------------------------------+
        | margin (above first row only)                 |
        |-----------------------------------------------|
        | vspace (hspace)                               |
        |-----------------------------------------------|
        | hpadding                                      |
        |-----------------------------------------------|
        | _  _  __ +--------+ _  __ +--------+ _  __  _ |
        ||m||h|| h|| Widget ||h|| h|| Widget ||h|| h||m||
        ||a||s|| p||        ||s|| p||        ||s|| p||a||
        ||r||p|| a||        ||p|| a||        ||p|| a||r||
        ||g||a|| d||        ||a|| d||        ||a|| d||g||
        ||i||c|| d||        ||c|| d||        ||c|| d||i||
        ||n||e|| i||        ||e|| i||        ||e|| i||n||
        || || || n||        || || n||        || || n|| ||
        || || || g||        || || g||        || || g|| ||
        | -  -  -- +--------+ -  -- +--------+ -  --  - |
        |-----------------------------------------------|
        | vspace (hspace)                               |
        |-----------------------------------------------|
        | hpadding                                      |
        |-----------------------------------------------|
        | margin (below last row only)                  |
        +-----------------------------------------------+

        margin:   doesn't play a role in the code below, it only affects the
                  effective rectangle inside which we can layout
        hspace:   static padding between widgets (minimum distance between them)
        hpadding: dynamic padding to fill the space when resizing
        </pre>

        @param self: object self-reference
        @param rect: the area the widgets should occupy
        @param testonly: only test the layout, don't arrange the items

        @return: the height of the layout
        """

        left, top, right, bottom = self.getContentsMargins()
        effective = rect.adjusted(+left, +top, -right, -bottom)
        x = effective.x()
        y = effective.y()
        lineheight = 0

        if not self._items:
            return y + lineheight - rect.y() + bottom

        widget = self._items[0].widget()
        hspace = self.horizontalSpacing()
        if hspace == -1:
            hspace = widget.style().layoutSpacing(
                QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Horizontal
            )
        vspace = self.verticalSpacing()
        if vspace == -1:
            vspace = widget.style().layoutSpacing(QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Vertical)

        # lk: get the remaining space after subtracting the space required for each widget and its static padding
        # lk: also reserve space for the leading static padding
        rem_hspace = (effective.width() - hspace) % (widget.size().width() + hspace)
        # lk: the number of items and their static spacing that can be in the layout
        hspace_items = (effective.width() - hspace) // (widget.size().width() + hspace)

        # lk: in case the visible items are less than the maximum possible widgets
        visible_items = len([item for item in self._items if not item.isEmpty()])
        if visible_items < hspace_items:
            hspace_items = visible_items
            rem_hspace = (effective.width() - hspace) - ((widget.size().width() + hspace) * hspace_items)

        try:
            # lk: the dynamic padding between each item, also account for the leading dynamic padding
            hpadding = rem_hspace // (hspace_items + 1)
        except ZeroDivisionError:
            hpadding = 0

        for item in self._items:
            if item.isEmpty():
                continue
            # lk: compute the location of the next widget
            next_x = x + item.sizeHint().width() + hspace + hpadding
            # lk: find out if there is enough space for the widget in this row
            # lk: account for the leading static and dynamic padding too (at the start)
            if next_x - hspace * 2 - hpadding * 2 > effective.right() and lineheight > 0:
                x = effective.x()
                # lk: find next vertical position, add static and dynamic padding
                y = y + lineheight + vspace + hpadding
                next_x = x + item.sizeHint().width() + hspace + hpadding
                lineheight = 0
            # lk: add static and dynamic padding to the current widget
            x = x + hspace + hpadding
            if not testonly:
                item.setGeometry(QRect(QPoint(x, y), item.sizeHint()))
            x = next_x
            lineheight = max(lineheight, item.sizeHint().height())
        return y + lineheight - rect.y() + bottom

    def sort(self, key: Callable, reverse=False) -> None:
        self._items.sort(key=key, reverse=reverse)
        self.setGeometry(self.parent().contentsRect().adjusted(*self.parent().getContentsMargins()))



import os
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import List, Optional, Dict, Tuple

from legendary.models.downloading import AnalysisResult, ConditionCheckResult
from legendary.models.game import Game, InstalledGame

from rare.lgndr.downloader.mp.manager import DLManager


@dataclass
class InstallOptionsModel:
    app_name: str
    base_path: str = ""
    shared_memory: int = 1024
    max_workers: int = os.cpu_count() * 2
    force: bool = False
    platform: str = "Windows"
    install_tag: Optional[List[str]] = None
    order_opt: bool = False
    repair_mode: bool = False
    repair_and_update: bool = False
    no_install: bool = False
    ignore_space: bool = False
    reset_sdl: bool = False
    skip_dlcs: bool = False
    with_dlcs: bool = False
    # Rare's internal arguments
    # FIXME: Do we really need all of these?
    create_shortcut: bool = True
    overlay: bool = False
    update: bool = False
    silent: bool = False
    install_prereqs: bool = False

    def as_install_kwargs(self) -> Dict:
        return {
            k: getattr(self, k)
            for k in vars(self)
            if k not in ["update", "silent", "create_shortcut", "overlay", "install_prereqs"]
        }


@dataclass
class InstallDownloadModel:
    dlm: DLManager
    analysis: AnalysisResult
    igame: InstalledGame
    game: Game
    repair: bool
    repair_file: str
    res: ConditionCheckResult


class InstallQueueItemModel:
    def __init__(self, options: InstallOptionsModel,  download: InstallDownloadModel = None):
        self.options: Optional[InstallOptionsModel] = options
        # lk: internal attribute holders
        self.__download: Optional[InstallDownloadModel] = None
        self.__date: Optional[datetime] = None

        self.download = download

    @property
    def download(self) -> Optional[InstallDownloadModel]:
        return self.__download

    @download.setter
    def download(self, download: Optional[InstallDownloadModel]):
        self.__download = download
        if download is not None:
            self.__date = datetime.now()

    @property
    def expired(self) -> bool:
        return datetime.now() > (self.__date + timedelta(minutes=30))

    def __bool__(self):
        return (self.download is not None) and (self.options is not None) and (not self.expired)


@dataclass
class UninstallOptionsModel:
    app_name: str
    accepted: bool = None
    keep_files: bool = None
    keep_config: bool = None

    def __bool__(self):
        return (
            bool(self.app_name)
            and (self.accepted is not None)
            and (self.keep_files is not None)
            and (self.keep_config is not None)
        )

    @property
    def values(self) -> Tuple[bool, bool, bool]:
        """
        This model's options

        :return:
            Tuple of `accepted` `keep_files` `keep_config`
        """
        return self.accepted, self.keep_config, self.keep_files

    @values.setter
    def values(self, values: Tuple[bool, bool, bool]):
        """
        Set this model's options

        :param values:
            Tuple of `accepted` `keep_files` `keep_config`
        :return:
        """
        self.accepted = values[0]
        self.keep_files = values[1]
        self.keep_config = values[2]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/game_info/cloud_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CloudWidget(object):
    def setupUi(self, CloudWidget):
        CloudWidget.setObjectName("CloudWidget")
        CloudWidget.resize(251, 93)
        CloudWidget.setWindowTitle("GroupBox")
        self.cloud_layout = QtWidgets.QFormLayout(CloudWidget)
        self.cloud_layout.setObjectName("cloud_layout")
        self.cloud_sync_label = QtWidgets.QLabel(CloudWidget)
        self.cloud_sync_label.setObjectName("cloud_sync_label")
        self.cloud_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.cloud_sync_label)
        self.cloud_sync = QtWidgets.QCheckBox(CloudWidget)
        self.cloud_sync.setText("")
        self.cloud_sync.setObjectName("cloud_sync")
        self.cloud_layout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cloud_sync)

        self.retranslateUi(CloudWidget)

    def retranslateUi(self, CloudWidget):
        _translate = QtCore.QCoreApplication.translate
        CloudWidget.setTitle(_translate("CloudWidget", "Options"))
        self.cloud_sync_label.setText(_translate("CloudWidget", "Sync with cloud"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    CloudWidget = QtWidgets.QGroupBox()
    ui = Ui_CloudWidget()
    ui.setupUi(CloudWidget)
    CloudWidget.show()
    sys.exit(app.exec_())



from PyQt5 import QtGui
from PyQt5.QtCore import pyqtSignal, Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
    QScrollArea,
    QGroupBox,
    QPushButton,
    QStackedWidget,
)

from rare.utils.extra_widgets import ImageLabel, WaitingSpinner
from rare.widgets.flow_layout import FlowLayout


class SearchResults(QStackedWidget):
    show_info = pyqtSignal(dict)

    def __init__(self, api_core):
        super(SearchResults, self).__init__()
        self.search_result_widget = QWidget()
        self.api_core = api_core
        self.addWidget(self.search_result_widget)
        self.main_layout = QVBoxLayout()
        self.back_button = QPushButton(self.tr("Back"))
        self.main_layout.addWidget(self.back_button)
        self.main_layout.addWidget(self.back_button)
        self.result_area = QScrollArea()
        self.widget = QWidget()
        self.result_area.setWidgetResizable(True)
        self.main_layout.addWidget(self.result_area)
        self.result_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        self.result_area.setWidget(self.widget)
        self.layout = FlowLayout()
        self.widget.setLayout(self.layout)

        self.search_result_widget.setLayout(self.main_layout)

        self.addWidget(WaitingSpinner())
        self.setCurrentIndex(1)

    def load_results(self, text: str):
        self.setCurrentIndex(1)
        if text != "":
            self.api_core.search_game(text, self.show_results)

    def show_results(self, results: dict):
        self.widget.deleteLater()
        self.widget = QWidget()
        self.layout = FlowLayout()
        if not results:
            self.layout.addWidget(QLabel(self.tr("No results found")))
        else:
            for res in results:
                w = _SearchResultItem(res)
                w.show_info.connect(self.show_info.emit)
                self.layout.addWidget(w)
        self.widget.setLayout(self.layout)
        self.result_area.setWidget(self.widget)
        self.setCurrentIndex(0)


class _SearchResultItem(QGroupBox):
    res: dict
    show_info = pyqtSignal(dict)

    def __init__(self, result: dict):
        super(_SearchResultItem, self).__init__()
        self.layout = QVBoxLayout()
        self.image = ImageLabel()
        for img in result["keyImages"]:
            if img["type"] == "DieselStoreFrontTall":
                width = 240
                self.image.update_image(img["url"], result["title"], (width, 360))
                break
        else:
            print("No image found")
        self.layout.addWidget(self.image)

        self.res = result
        self.title = QLabel(self.res["title"])
        title_font = QFont()
        title_font.setPixelSize(15)
        self.title.setFont(title_font)
        self.title.setWordWrap(True)
        self.layout.addWidget(self.title)
        price = result["price"]["totalPrice"]["fmtPrice"]["originalPrice"]
        discount_price = result["price"]["totalPrice"]["fmtPrice"]["discountPrice"]
        price_layout = QHBoxLayout()
        price_label = QLabel(price if price != "0" else self.tr("Free"))
        price_layout.addWidget(price_label)

        if price != discount_price:
            font = QFont()
            font.setStrikeOut(True)
            price_label.setFont(font)
            price_layout.addWidget(QLabel(discount_price))
        # self.discount_price = QLabel(f"{self.tr('Discount price: ')}{discount_price}")
        self.layout.addLayout(price_layout)

        self.setLayout(self.layout)

        self.setFixedWidth(260)

    def mousePressEvent(self, a0: QtGui.QMouseEvent) -> None:
        if a0.button() == 1:
            self.show_info.emit(self.res)



import json
from logging import getLogger
from typing import Tuple

from PyQt5.QtCore import pyqtSignal, QUrl
from PyQt5.QtGui import QDesktopServices
from PyQt5.QtWidgets import QFrame, qApp, QFormLayout, QLineEdit
from legendary.core import LegendaryCore
from legendary.utils import webview_login

from rare.ui.components.dialogs.login.browser_login import Ui_BrowserLogin
from rare.utils.misc import icon
from rare.widgets.indicator_edit import IndicatorLineEdit, IndicatorReasonsCommon

logger = getLogger("BrowserLogin")


class BrowserLogin(QFrame):
    success = pyqtSignal()
    changed = pyqtSignal()

    def __init__(self, core: LegendaryCore, parent=None):
        super(BrowserLogin, self).__init__(parent=parent)
        self.setFrameStyle(self.StyledPanel)
        self.ui = Ui_BrowserLogin()
        self.ui.setupUi(self)

        self.core = core
        self.login_url = self.core.egs.get_auth_url()

        self.sid_edit = IndicatorLineEdit(
            placeholder=self.tr("Insert authorizationCode here"), edit_func=self.text_changed, parent=self
        )
        self.sid_edit.line_edit.setEchoMode(QLineEdit.Password)
        self.ui.link_text.setText(self.login_url)
        self.ui.copy_button.setIcon(icon("mdi.content-copy", "fa.copy"))
        self.ui.copy_button.clicked.connect(self.copy_link)
        self.ui.form_layout.setWidget(
            self.ui.form_layout.getWidgetPosition(self.ui.sid_label)[0],
            QFormLayout.FieldRole, self.sid_edit
        )

        self.ui.open_button.clicked.connect(self.open_browser)
        self.sid_edit.textChanged.connect(self.changed.emit)

    def copy_link(self):
        clipboard = qApp.clipboard()
        clipboard.setText(self.login_url)
        self.ui.status_label.setText(self.tr("Copied to clipboard"))

    def is_valid(self):
        return self.sid_edit.is_valid

    @staticmethod
    def text_changed(text) -> Tuple[bool, str, int]:
        if text:
            text = text.strip()
            if text.startswith("{") and text.endswith("}"):
                try:
                    text = json.loads(text).get("authorizationCode")
                except json.JSONDecodeError:
                    return False, text, IndicatorReasonsCommon.WRONG_FORMAT
            elif '"' in text:
                text = text.strip('"')
            return len(text) == 32, text, IndicatorReasonsCommon.WRONG_FORMAT
        else:
            return False, text, IndicatorReasonsCommon.VALID

    def do_login(self):
        self.ui.status_label.setText(self.tr("Logging in..."))
        auth_code = self.sid_edit.text()
        try:
            if self.core.auth_code(auth_code):
                logger.info("Successfully logged in as %s", self.core.lgd.userdata['displayName'])
                self.success.emit()
            else:
                self.ui.status_label.setText(self.tr("Login failed."))
                logger.warning("Failed to login through browser")
        except Exception as e:
            logger.warning(e)

    def open_browser(self):
        if not webview_login.webview_available:
            logger.warning("You don't have webengine installed, you will need to manually copy the authorizationCode.")
            QDesktopServices.openUrl(QUrl(self.login_url))
        else:
            if webview_login.do_webview_login(callback_code=self.core.auth_ex_token):
                logger.info("Successfully logged in as %s", {self.core.lgd.userdata['displayName']})
                self.success.emit()
            else:
                logger.warning("Failed to login through browser.")



import json
import os
from dataclasses import dataclass
from enum import IntEnum
from logging import getLogger
from pathlib import Path
from typing import List, Tuple, Optional, Set

from PyQt5.QtCore import Qt, QModelIndex, pyqtSignal, QRunnable, QObject, QThreadPool, pyqtSlot
from PyQt5.QtGui import QStandardItemModel, QShowEvent
from PyQt5.QtWidgets import (
    QFileDialog,
    QGroupBox,
    QCompleter,
    QTreeView,
    QHeaderView,
    QMessageBox,
    QStackedWidget,
    QProgressBar,
    QSizePolicy,
    QFormLayout,
)

from rare.lgndr.cli import LegendaryCLI
from rare.lgndr.core import LegendaryCore
from rare.lgndr.glue.arguments import LgndrImportGameArgs
from rare.lgndr.glue.monkeys import LgndrIndirectStatus, get_boolean_choice_factory
from rare.shared import RareCore
from rare.ui.components.tabs.games.integrations.import_group import Ui_ImportGroup
from rare.widgets.elide_label import ElideLabel
from rare.widgets.indicator_edit import IndicatorLineEdit, IndicatorReasonsCommon, PathEdit

logger = getLogger("Import")


def find_app_name(path: str, core) -> Optional[str]:
    if os.path.exists(os.path.join(path, ".egstore")):
        for i in os.listdir(os.path.join(path, ".egstore")):
            if i.endswith(".mancpn"):
                with open(os.path.join(path, ".egstore", i)) as file:
                    app_name = json.load(file).get("AppName")
                return app_name
    elif app_name := LegendaryCLI(core).resolve_aliases(os.path.basename(os.path.normpath(path))):
        # return None if game does not exist (Workaround for overlay)
        if not core.get_game(app_name):
            return None
        return app_name
    else:
        logger.warning(f"Could not find AppName for {path}")
    return None


class ImportResult(IntEnum):
    ERROR = 0
    FAILED = 1
    SUCCESS = 2


@dataclass
class ImportedGame:
    result: ImportResult
    path: Optional[str] = None
    app_name: Optional[str] = None
    app_title: Optional[str] = None
    message: Optional[str] = None


class ImportWorker(QRunnable):
    class Signals(QObject):
        progress = pyqtSignal(ImportedGame, int)
        result = pyqtSignal(list)

    def __init__(
            self,
            core: LegendaryCore,
            path: str,
            app_name: str = None,
            platform: Optional[str] = None,
            import_folder: bool = False,
            import_dlcs: bool = False,
            import_force: bool = False
    ):
        super(ImportWorker, self).__init__()
        self.setAutoDelete(True)
        self.signals = ImportWorker.Signals()
        self.core = core

        self.path = Path(path)
        self.app_name = app_name
        self.import_folder = import_folder
        self.platform = platform if platform is not None else self.core.default_platform
        self.import_dlcs = import_dlcs
        self.import_force = import_force

    def run(self) -> None:
        result_list: List = []
        if self.import_folder:
            folders = [i for i in self.path.iterdir() if i.is_dir()]
            number_of_folders = len(folders)
            for i, child in enumerate(folders):
                if not child.is_dir():
                    continue
                result = self.__try_import(child, None)
                result_list.append(result)
                self.signals.progress.emit(result, int(100 * i // number_of_folders))
        else:
            result = self.__try_import(self.path, self.app_name)
            result_list.append(result)
            self.signals.progress.emit(result, 100)
        self.signals.result.emit(result_list)

    def __try_import(self, path: Path, app_name: str = None) -> ImportedGame:
        result = ImportedGame(ImportResult.ERROR)
        result.path = str(path)
        if app_name or (app_name := find_app_name(str(path), self.core)):
            game = self.core.get_game(app_name)
            result.app_name = app_name
            result.app_title = game.app_title
            platform = self.platform
            if platform not in self.core.get_game(app_name, update_meta=False).asset_infos:
                platform = "Windows"
            success, message = self.__import_game(path, app_name, platform)
            if not success:
                result.result = ImportResult.FAILED
                result.message = message
            else:
                result.result = ImportResult.SUCCESS
        return result

    def __import_game(self, path: Path, app_name: str, platform: str):
        cli = LegendaryCLI(self.core)
        status = LgndrIndirectStatus()
        args = LgndrImportGameArgs(
            app_path=str(path),
            app_name=app_name,
            platform=platform,
            disable_check=self.import_force,
            skip_dlcs=not self.import_dlcs,
            with_dlcs=self.import_dlcs,
            indirect_status=status,
            get_boolean_choice=get_boolean_choice_factory(self.import_dlcs)
        )
        cli.import_game(args)
        return status.success, status.message


class AppNameCompleter(QCompleter):
    activated = pyqtSignal(str)

    def __init__(self, app_names: Set[Tuple[str, str]], parent=None):
        super(AppNameCompleter, self).__init__(parent)
        # pylint: disable=E1136
        super(AppNameCompleter, self).activated[QModelIndex].connect(self.__activated_idx)

        model = QStandardItemModel(len(app_names), 2)
        for idx, game in enumerate(app_names):
            app_name, app_title = game
            model.setData(model.index(idx, 0), app_title)
            model.setData(model.index(idx, 1), app_name)
        self.setModel(model)

        treeview = QTreeView()
        self.setPopup(treeview)
        treeview.setRootIsDecorated(False)
        treeview.header().hide()
        treeview.header().setSectionResizeMode(0, QHeaderView.Stretch)
        treeview.header().setSectionResizeMode(1, QHeaderView.Stretch)

        # listview = QListView()
        # self.setPopup(listview)
        # # listview.setModelColumn(1)

        self.setFilterMode(Qt.MatchContains)
        self.setCaseSensitivity(Qt.CaseInsensitive)
        # self.setCompletionMode(QCompleter.UnfilteredPopupCompletion)

    def __activated_idx(self, idx):
        # lk: don't even look at this in a funny way, it will die of shame
        # lk: Note to self, the completer and popup models are different.
        # lk: Getting the index from the popup and trying to use it in the completer will return invalid results
        if isinstance(idx, QModelIndex):
            self.activated.emit(self.popup().model().data(self.popup().model().index(idx.row(), 1)))
        # TODO: implement conversion from app_name to app_title (signal loop here)
        # if isinstance(idx_str, str):
        #     self.activated.emit(idx_str)


class ImportGroup(QGroupBox):
    def __init__(self, parent=None):
        super(ImportGroup, self).__init__(parent=parent)
        self.ui = Ui_ImportGroup()
        self.ui.setupUi(self)
        self.rcore = RareCore.instance()
        self.core = RareCore.instance().core()

        self.worker: Optional[ImportWorker] = None
        self.threadpool = QThreadPool.globalInstance()

        self.__app_names: Set[str] = set()
        self.__install_dirs: Set[str] = set()

        self.path_edit = PathEdit(
            self.core.get_default_install_dir(self.core.default_platform),
            QFileDialog.DirectoryOnly,
            edit_func=self.path_edit_callback,
            parent=self,
        )
        self.path_edit.textChanged.connect(self.path_changed)
        self.ui.import_layout.setWidget(
            self.ui.import_layout.getWidgetPosition(self.ui.path_edit_label)[0],
            QFormLayout.FieldRole, self.path_edit
        )

        self.app_name_edit = IndicatorLineEdit(
            placeholder=self.tr("Use in case the app name was not found automatically"),
            edit_func=self.app_name_edit_callback,
            save_func=self.app_name_save_callback,
            parent=self,
        )
        self.app_name_edit.textChanged.connect(self.app_name_changed)
        self.ui.import_layout.setWidget(
            self.ui.import_layout.getWidgetPosition(self.ui.app_name_label)[0],
            QFormLayout.FieldRole, self.app_name_edit
        )

        self.ui.import_folder_check.stateChanged.connect(self.import_folder_changed)
        self.ui.import_dlcs_check.setEnabled(False)
        self.ui.import_dlcs_check.stateChanged.connect(self.import_dlcs_changed)

        self.ui.import_button_label.setText("")
        self.ui.import_button.setEnabled(False)
        self.ui.import_button.clicked.connect(
            lambda: self.__import(self.path_edit.text())
        )

        self.button_info_stack = QStackedWidget(self)
        self.button_info_stack.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.button_info_stack.setFixedHeight(self.ui.import_button.sizeHint().height())
        self.info_label = ElideLabel(text="", parent=self.button_info_stack)
        self.info_label.setFixedHeight(False)
        self.info_label.setAlignment(Qt.AlignVCenter)
        self.info_progress = QProgressBar(self.button_info_stack)
        self.button_info_stack.addWidget(self.info_label)
        self.button_info_stack.addWidget(self.info_progress)
        self.ui.button_info_layout.insertWidget(0, self.button_info_stack)

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.__app_names = {rgame.app_name for rgame in self.rcore.games}
        self.__install_dirs = {rgame.folder_name for rgame in self.rcore.games if not rgame.is_dlc}
        self.app_name_edit.setCompleter(
            AppNameCompleter(app_names={(rgame.app_name, rgame.app_title) for rgame in self.rcore.games})
        )
        super().showEvent(a0)

    def set_game(self, app_name: str):
        if app_name:
            rgame = self.rcore.get_game(app_name)
            self.path_edit.setText(
                os.path.join(self.core.get_default_install_dir(rgame.default_platform), rgame.folder_name)
            )
            self.app_name_edit.setText(app_name)

    def path_edit_callback(self, path) -> Tuple[bool, str, int]:
        if os.path.exists(path):
            if os.path.exists(os.path.join(path, ".egstore")):
                return True, path, IndicatorReasonsCommon.VALID
            elif os.path.basename(path) in self.__install_dirs:
                return True, path, IndicatorReasonsCommon.VALID
        else:
            return False, path, IndicatorReasonsCommon.DIR_NOT_EXISTS
        return False, path, IndicatorReasonsCommon.UNDEFINED

    @pyqtSlot(str)
    def path_changed(self, path: str):
        self.info_label.setText("")
        self.ui.import_folder_check.setCheckState(Qt.Unchecked)
        self.ui.import_force_check.setCheckState(Qt.Unchecked)
        if self.path_edit.is_valid:
            self.app_name_edit.setText(find_app_name(path, self.core))
        else:
            self.app_name_edit.setText("")

    def app_name_edit_callback(self, text) -> Tuple[bool, str, int]:
        if not text:
            return False, text, IndicatorReasonsCommon.UNDEFINED
        if text in self.__app_names:
            return True, text, IndicatorReasonsCommon.VALID
        else:
            return False, text, IndicatorReasonsCommon.NOT_INSTALLED

    def app_name_save_callback(self, text) -> None:
        rgame = self.rcore.get_game(text)
        self.ui.platform_combo.clear()
        self.ui.platform_combo.addItems(rgame.platforms)
        self.ui.platform_combo.setCurrentText(rgame.default_platform)

    @pyqtSlot(str)
    def app_name_changed(self, app_name: str):
        self.info_label.setText("")
        self.ui.import_dlcs_check.setCheckState(Qt.Unchecked)
        self.ui.import_force_check.setCheckState(Qt.Unchecked)
        self.ui.import_dlcs_check.setEnabled(
            self.app_name_edit.is_valid and bool(self.core.get_dlc_for_game(app_name))
        )
        self.ui.import_button.setEnabled(
            not bool(self.worker) and (self.app_name_edit.is_valid and self.path_edit.is_valid)
        )

    @pyqtSlot(int)
    def import_folder_changed(self, state: Qt.CheckState):
        self.app_name_edit.setEnabled(not state)
        self.ui.platform_combo.setEnabled(not state)
        self.ui.platform_combo.setToolTip(
            self.tr(
                "When importing multiple games, the current OS will be used at the"
                " platform for the games that support it, otherwise the Windows version"
                " will be imported."
            ) if state else ""
        )
        self.ui.import_dlcs_check.setCheckState(Qt.Unchecked)
        self.ui.import_force_check.setCheckState(Qt.Unchecked)
        self.ui.import_dlcs_check.setEnabled(
            state
            or (self.app_name_edit.is_valid and bool(self.core.get_dlc_for_game(self.app_name_edit.text())))
        )
        self.ui.import_button.setEnabled(
            not bool(self.worker) and (state or (not state and self.app_name_edit.is_valid))
        )

    @pyqtSlot(int)
    def import_dlcs_changed(self, state: Qt.CheckState):
        self.ui.import_button.setEnabled(
            not bool(self.worker) and (self.ui.import_folder_check.isChecked() or self.app_name_edit.is_valid)
        )

    @pyqtSlot(str)
    def __import(self, path: Optional[str] = None):
        self.ui.import_button.setDisabled(True)
        self.info_label.setText(self.tr("Status: Importing games"))
        self.info_progress.setValue(0)
        self.button_info_stack.setCurrentWidget(self.info_progress)

        if not path:
            path = self.path_edit.text()
        self.worker = ImportWorker(
            self.core,
            path,
            app_name=self.app_name_edit.text(),
            platform=self.ui.platform_combo.currentText() if not self.ui.import_folder_check.isChecked() else None,
            import_folder=self.ui.import_folder_check.isChecked(),
            import_dlcs=self.ui.import_dlcs_check.isChecked(),
            import_force=self.ui.import_force_check.isChecked()
        )
        self.worker.signals.progress.connect(self.__on_import_progress)
        self.worker.signals.result.connect(self.__on_import_result)
        self.threadpool.start(self.worker)

    @pyqtSlot(ImportedGame, int)
    def __on_import_progress(self, imported: ImportedGame, progress: int):
        self.info_progress.setValue(progress)
        if imported.result == ImportResult.SUCCESS:
            self.rcore.get_game(imported.app_name).set_installed(True)
        status = "error" if not imported.result else (
            "failed" if imported.result == ImportResult.FAILED else "successful"
        )
        logger.info(f"Import {status}: {imported.app_title}: {imported.path} ({imported.message})")

    @pyqtSlot(list)
    def __on_import_result(self, result: List[ImportedGame]):
        self.worker = None
        self.button_info_stack.setCurrentWidget(self.info_label)
        if len(result) == 1:
            res = result[0]
            if res.result == ImportResult.SUCCESS:
                self.info_label.setText(
                    self.tr("Success: <b>{}</b> imported").format(res.app_title)
                )
            elif res.result == ImportResult.FAILED:
                self.info_label.setText(
                    self.tr("Failed: <b>{}</b> - {}").format(res.app_title, res.message)
                )
            else:
                self.info_label.setText(
                    self.tr("Error: Could not find AppName for <b>{}</b>").format(res.path)
                )
        else:
            self.info_label.setText(self.tr("Status: Finished importing games"))
            success = [r for r in result if r.result == ImportResult.SUCCESS]
            failure = [r for r in result if r.result == ImportResult.FAILED]
            errored = [r for r in result if r.result == ImportResult.ERROR]
            # pylint: disable=E1101
            messagebox = QMessageBox(
                QMessageBox.Information,
                self.tr("Import summary"),
                self.tr(
                    "Tried to import {} folders.\n\n"
                    "Successfully imported {} games, failed to import {} games and {} errors occurred"
                ).format(len(success) + len(failure) + len(errored), len(success), len(failure), len(errored)),
                buttons=QMessageBox.StandardButton.Close,
                parent=self,
            )
            messagebox.setWindowModality(Qt.NonModal)
            details: List = []
            for res in success:
                details.append(
                    self.tr("Success: {} imported").format(res.app_title)
                )
            for res in failure:
                details.append(
                    self.tr("Failed: {} - {}").format(res.app_title, res.message)
                )
            for res in errored:
                details.append(
                    self.tr("Error: Could not find AppName for {}").format(res.path)
                )
            messagebox.setDetailedText("\n".join(details))
            messagebox.show()



import os
import platform
import shutil
from argparse import Namespace
from dataclasses import dataclass
from logging import getLogger
from typing import List

from PyQt5.QtCore import QProcess, QProcessEnvironment
from legendary.models.game import InstalledGame, LaunchParameters

from rare.lgndr.core import LegendaryCore


logger = getLogger("Helper")


class GameArgsError(Exception):
    pass


class InitArgs(Namespace):
    app_name: str
    dry_run: bool = False
    debug: bool = False
    offline: bool = False
    skip_update_check: bool = False
    wine_prefix: str = ""
    wine_bin: str = ""

    @classmethod
    def from_argparse(cls, args):
        return cls(
            app_name=args.app_name,
            debug=args.debug,
            offline=args.offline,
            skip_update_check=args.skip_update_check,
            wine_bin=args.wine_bin,
            wine_prefix=args.wine_pfx,
            dry_run=args.dry_run
        )


@dataclass
class LaunchArgs:
    executable: str = ""
    arguments: List[str] = None
    working_directory: str = None
    environment: QProcessEnvironment = None
    pre_launch_command: str = ""
    pre_launch_wait: bool = False
    is_origin_game: bool = False  # only for windows to launch as url

    def __bool__(self):
        return bool(self.executable)


def get_origin_params(core: LegendaryCore, app_name, offline: bool,
                      launch_args: LaunchArgs) -> LaunchArgs:
    origin_uri = core.get_origin_uri(app_name, offline)
    if platform.system() == "Windows":
        launch_args.executable = origin_uri
        launch_args.arguments = []
        # only set it here true, because on linux it is a launch command like every other game
        launch_args.is_origin_game = True
        return launch_args

    command = core.get_app_launch_command(app_name)
    if not os.path.exists(command[0]) and shutil.which(command[0]) is None:
        return launch_args
    command.append(origin_uri)

    env = core.get_app_environment(app_name)
    launch_args.environment = QProcessEnvironment.systemEnvironment()

    if os.environ.get("container") == "flatpak":
        flatpak_command = ["flatpak-spawn", "--host"]
        for name, value in env.items():
            flatpak_command.append(f"--env={name}={value}")
        command = flatpak_command + command
    else:
        for name, value in env.items():
            launch_args.environment.insert(name, value)

    launch_args.executable = command[0]
    launch_args.arguments = command[1:]

    return launch_args


def get_game_params(core: LegendaryCore, igame: InstalledGame, args: InitArgs,
                    launch_args: LaunchArgs) -> LaunchArgs:
    if not args.offline:  # skip for update
        if not args.skip_update_check and not core.is_noupdate_game(igame.app_name):
            # print("Checking for updates...")
            # check updates
            try:
                latest = core.get_asset(
                    igame.app_name, igame.platform, update=False
                )
            except ValueError:
                raise GameArgsError("Metadata doesn't exist")
            else:
                if latest.build_version != igame.version:
                    raise GameArgsError("Game is not up to date. Please update first")

    params: LaunchParameters = core.get_launch_parameters(
        app_name=igame.app_name, offline=args.offline
    )

    full_params = []
    launch_args.environment = QProcessEnvironment.systemEnvironment()

    if os.environ.get("container") == "flatpak":
        full_params.extend(["flatpak-spawn", "--host"])
        for name, value in params.environment.items():
            full_params.append(f"--env={name}={value}")
    else:
        for name, value in params.environment.items():
            launch_args.environment.insert(name, value)

    full_params.extend(params.launch_command)
    full_params.append(
        os.path.join(params.game_directory, params.game_executable)
    )
    full_params.extend(params.game_parameters)
    full_params.extend(params.egl_parameters)
    full_params.extend(params.user_parameters)

    launch_args.executable = full_params[0]
    launch_args.arguments = full_params[1:]
    launch_args.working_directory = params.working_directory

    return launch_args


def get_launch_args(core: LegendaryCore, args: InitArgs = None) -> LaunchArgs:
    game = core.get_game(args.app_name)
    igame = core.get_installed_game(args.app_name)

    resp = LaunchArgs()

    if not game:
        raise GameArgsError(f"Could not find metadata for ")

    if game.third_party_store == "Origin":
        args.offline = False
    else:
        if not igame:
            raise GameArgsError("Game is not installed or has unsupported format")

        if game.is_dlc:
            raise GameArgsError("Game is a DLC")
        if not os.path.exists(igame.install_path):
            raise GameArgsError("Game path does not exist")

    if game.third_party_store == "Origin":
        resp = get_origin_params(core, args.app_name, args.offline, resp)
    else:
        resp = get_game_params(core, igame, args, resp)

    pre_cmd, wait = core.get_pre_launch_command(args.app_name)
    resp.pre_launch_command, resp.pre_launch_wait = pre_cmd, wait
    return resp


def get_configured_process(env: dict = None):
    proc = QProcess()
    proc.setProcessChannelMode(QProcess.MergedChannels)
    proc.readyReadStandardOutput.connect(
        lambda: logger.info(
            str(proc.readAllStandardOutput().data(), "utf-8", "ignore")
        )
    )
    environment = QProcessEnvironment.systemEnvironment()
    if env:
        for e in env:
            environment.insert(e, env[e])
    proc.setProcessEnvironment(environment)

    return proc



from typing import Optional, List, overload

from PyQt5.QtCore import Qt, QRect, QSize, QPoint
from PyQt5.QtWidgets import QLayout, QStyle, QSizePolicy, QLayoutItem, QWidget


class FlowLayout(QLayout):
    def __init__(self, parent=None):
        super(FlowLayout, self).__init__(parent)
        self.setObjectName(type(self).__name__)
        self._hspacing = -1
        self._vspacing = -1
        self._items: List[QLayoutItem] = []

    def __del__(self):
        del self._items[:]

    @overload
    def indexOf(self, a0: QWidget) -> int:
        try:
            return next(idx for idx, item in enumerate(self._items) if item.widget() is a0)
        except:
            return -1

    def indexOf(self, a0: QLayoutItem) -> int:
        try:
            return self._items.index(a0)
        except:
            return -1

    def addItem(self, a0: QLayoutItem) -> None:
        self._items.append(a0)
        self.invalidate()

    def removeItem(self, a0: QLayoutItem) -> None:
        self._items.remove(a0)
        self.invalidate()

    def spacing(self) -> int:
        hspacing = self.horizontalSpacing()
        if hspacing == self.verticalSpacing():
            return hspacing
        else:
            return -1

    def setSpacing(self, a0: int) -> None:
        self._hspacing = self._vspacing = a0
        self.invalidate()

    def setHorizontalSpacing(self, a0: int) -> None:
        self._hspacing = a0
        self.invalidate()

    def horizontalSpacing(self):
        if self._hspacing >= 0:
            return self._hspacing
        else:
            return self.smartSpacing(QStyle.PM_LayoutHorizontalSpacing)

    def setVerticalSpacing(self, a0:  int) -> None:
        self._vspacing = a0
        self.invalidate()

    def verticalSpacing(self):
        if self._vspacing >= 0:
            return self._vspacing
        else:
            return self.smartSpacing(QStyle.PM_LayoutVerticalSpacing)

    def count(self) -> int:
        return len(self._items)

    def itemAt(self, index: int) -> Optional[QLayoutItem]:
        if 0 <= index < len(self._items):
            return self._items[index]
        return None

    def takeAt(self, index: int) -> Optional[QLayoutItem]:
        if 0 <= index < len(self._items):
            item = self._items.pop(index)
            self.invalidate()
            return item
        return None

    def expandingDirections(self) -> Qt.Orientations:
        return Qt.Orientations(Qt.Orientation(0))
        # return Qt.Horizontal | Qt.Vertical

    def hasHeightForWidth(self) -> bool:
        return True

    def heightForWidth(self, a0: int) -> int:
        return self.doLayout(QRect(0, 0, a0, 0), True)

    def setGeometry(self, a0: QRect) -> None:
        super(FlowLayout, self).setGeometry(a0)
        self.doLayout(a0, False)

    def sizeHint(self) -> QSize:
        return QSize(
            self.parent().contentsRect().size().width(),
            self.minimumSize().height()
        )

    def minimumSize(self) -> QSize:
        size = QSize()
        for item in self._items:
            size = size.expandedTo(item.minimumSize())
        left, top, right, bottom = self.getContentsMargins()
        size += QSize(left + right, top + bottom)
        return size

    def doLayout(self, rect, testonly):
        left, top, right, bottom = self.getContentsMargins()
        effective = rect.adjusted(+left, +top, -right, -bottom)
        x = effective.x()
        y = effective.y()
        lineheight = 0
        if not self._items:
            return y + lineheight - rect.y() + bottom
        for item in self._items:
            if item.isEmpty():
                continue
            widget = item.widget()
            hspace = self.horizontalSpacing()
            if hspace == -1:
                hspace = widget.style().layoutSpacing(
                    QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Horizontal
                )
            vspace = self.verticalSpacing()
            if vspace == -1:
                vspace = widget.style().layoutSpacing(
                    QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Vertical
                )
            nextX = x + item.sizeHint().width() + hspace
            if nextX - hspace > effective.right() and lineheight > 0:
                x = effective.x()
                y = y + lineheight + vspace
                nextX = x + item.sizeHint().width() + hspace
                lineheight = 0
            if not testonly:
                item.setGeometry(QRect(QPoint(x, y), item.sizeHint()))
            x = nextX
            lineheight = max(lineheight, item.sizeHint().height())
        return y + lineheight - rect.y() + bottom

    def smartSpacing(self, pm):
        parent = self.parent()
        if parent is None:
            return -1
        elif parent.isWidgetType():
            return parent.style().pixelMetric(pm, None, parent)
        else:
            return parent.spacing()



import webbrowser
from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtGui import QShowEvent
from PyQt5.QtWidgets import QWidget

from rare import __version__, __codename__
from rare.ui.components.tabs.settings.about import Ui_About
from rare.utils.qt_requests import QtRequests

logger = getLogger("About")


def versiontuple(v):
    try:
        return tuple(map(int, (v.split("."))))
    except Exception:
        return tuple((9, 9, 9))  # It is a beta version and newer


class About(QWidget):
    update_available_ready = pyqtSignal()

    def __init__(self, parent=None):
        super(About, self).__init__(parent=parent)
        self.ui = Ui_About()
        self.ui.setupUi(self)

        self.ui.version.setText(f"{__version__}  {__codename__}")

        self.ui.update_label.setEnabled(False)
        self.ui.update_lbl.setEnabled(False)
        self.ui.open_browser.setVisible(False)
        self.ui.open_browser.setEnabled(False)

        self.manager = QtRequests(parent=self)
        self.manager.get(
            "https://api.github.com/repos/RareDevs/Rare/releases/latest",
            self.update_available_finished,
        )

        self.ui.open_browser.clicked.connect(
            lambda: webbrowser.open("https://github.com/RareDevs/Rare/releases/latest")
        )

        self.update_available = False

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.manager.get(
            "https://api.github.com/repos/RareDevs/Rare/releases/latest",
            self.update_available_finished,
        )
        super().showEvent(a0)

    def update_available_finished(self, data: dict):
        if latest_tag := data.get("tag_name"):
            self.update_available = versiontuple(latest_tag) > versiontuple(__version__)
        else:
            self.update_available = False

        if self.update_available:
            logger.info(f"Update available: {__version__} -> {latest_tag}")
            self.ui.update_lbl.setText("{} -> {}".format(__version__, latest_tag))
            self.update_available_ready.emit()
        else:
            self.ui.update_lbl.setText(self.tr("You have the latest version"))
        self.ui.update_label.setEnabled(self.update_available)
        self.ui.update_lbl.setEnabled(self.update_available)
        self.ui.open_browser.setVisible(self.update_available)
        self.ui.open_browser.setEnabled(self.update_available)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/downloads/queue_info_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_QueueInfoWidget(object):
    def setupUi(self, QueueInfoWidget):
        QueueInfoWidget.setObjectName("QueueInfoWidget")
        QueueInfoWidget.resize(624, 60)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(QueueInfoWidget.sizePolicy().hasHeightForWidth())
        QueueInfoWidget.setSizePolicy(sizePolicy)
        QueueInfoWidget.setWindowTitle("QueueInfoWidget")
        self.queue_info_layout = QtWidgets.QGridLayout(QueueInfoWidget)
        self.queue_info_layout.setContentsMargins(0, 0, 0, 0)
        self.queue_info_layout.setObjectName("queue_info_layout")
        self.dl_size_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dl_size_label.setFont(font)
        self.dl_size_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dl_size_label.setObjectName("dl_size_label")
        self.queue_info_layout.addWidget(self.dl_size_label, 1, 3, 1, 1)
        self.title = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.title.setFont(font)
        self.title.setText("title")
        self.title.setObjectName("title")
        self.queue_info_layout.addWidget(self.title, 0, 1, 1, 4)
        self.local_version_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.local_version_label.setFont(font)
        self.local_version_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.local_version_label.setObjectName("local_version_label")
        self.queue_info_layout.addWidget(self.local_version_label, 2, 1, 1, 1)
        self.image_layout = QtWidgets.QVBoxLayout()
        self.image_layout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.image_layout.setObjectName("image_layout")
        self.queue_info_layout.addLayout(self.image_layout, 0, 0, 3, 1)
        self.install_size_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.install_size_label.setFont(font)
        self.install_size_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.install_size_label.setObjectName("install_size_label")
        self.queue_info_layout.addWidget(self.install_size_label, 2, 3, 1, 1)
        self.install_size = QtWidgets.QLabel(QueueInfoWidget)
        self.install_size.setMinimumSize(QtCore.QSize(160, 0))
        self.install_size.setText("install_size")
        self.install_size.setObjectName("install_size")
        self.queue_info_layout.addWidget(self.install_size, 2, 4, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.queue_info_layout.addItem(spacerItem, 0, 5, 3, 1)
        self.dl_size = QtWidgets.QLabel(QueueInfoWidget)
        self.dl_size.setMinimumSize(QtCore.QSize(160, 0))
        self.dl_size.setText("dl_size")
        self.dl_size.setObjectName("dl_size")
        self.queue_info_layout.addWidget(self.dl_size, 1, 4, 1, 1)
        self.remote_version_label = QtWidgets.QLabel(QueueInfoWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.remote_version_label.setFont(font)
        self.remote_version_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.remote_version_label.setObjectName("remote_version_label")
        self.queue_info_layout.addWidget(self.remote_version_label, 1, 1, 1, 1)
        self.local_version = QtWidgets.QLabel(QueueInfoWidget)
        self.local_version.setMinimumSize(QtCore.QSize(160, 0))
        self.local_version.setMaximumSize(QtCore.QSize(160, 16777215))
        self.local_version.setText("local_version")
        self.local_version.setObjectName("local_version")
        self.queue_info_layout.addWidget(self.local_version, 2, 2, 1, 1)
        self.remote_version = QtWidgets.QLabel(QueueInfoWidget)
        self.remote_version.setMinimumSize(QtCore.QSize(160, 0))
        self.remote_version.setMaximumSize(QtCore.QSize(160, 16777215))
        self.remote_version.setText("remote_version")
        self.remote_version.setObjectName("remote_version")
        self.queue_info_layout.addWidget(self.remote_version, 1, 2, 1, 1)

        self.retranslateUi(QueueInfoWidget)

    def retranslateUi(self, QueueInfoWidget):
        _translate = QtCore.QCoreApplication.translate
        self.dl_size_label.setText(_translate("QueueInfoWidget", "Download size"))
        self.local_version_label.setText(_translate("QueueInfoWidget", "Local version"))
        self.install_size_label.setText(_translate("QueueInfoWidget", "Installed size"))
        self.remote_version_label.setText(_translate("QueueInfoWidget", "Remote version"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    QueueInfoWidget = QtWidgets.QWidget()
    ui = Ui_QueueInfoWidget()
    ui.setupUi(QueueInfoWidget)
    QueueInfoWidget.show()
    sys.exit(app.exec_())



import datetime
import logging

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import (
    QGroupBox,
    QScrollArea,
    QCheckBox,
    QLabel,
    QPushButton,
    QHBoxLayout,
)

from legendary.core import LegendaryCore
from rare.ui.components.tabs.store.store import Ui_ShopWidget
from rare.utils.extra_widgets import WaitingSpinner, ButtonLineEdit
from rare.widgets.flow_layout import FlowLayout
from .constants import Constants
from .game_widgets import GameWidget
from .shop_api_core import ShopApiCore
from .shop_models import BrowseModel

logger = logging.getLogger("Shop")


# noinspection PyAttributeOutsideInit,PyBroadException
class ShopWidget(QScrollArea, Ui_ShopWidget):
    show_info = pyqtSignal(str)
    show_game = pyqtSignal(dict)
    free_game_widgets = []
    active_search_request = False
    next_search = ""
    wishlist: list = []

    def __init__(self, path, core: LegendaryCore, shop_api: ShopApiCore):
        super(ShopWidget, self).__init__()
        self.setWidgetResizable(True)
        self.setupUi(self)
        self.path = path
        self.core = core
        self.api_core = shop_api
        self.price = ""
        self.tags = []
        self.types = []
        self.update_games_allowed = True
        self.free_widget.setLayout(FlowLayout())

        self.free_stack.addWidget(WaitingSpinner())
        self.free_stack.setCurrentIndex(1)

        self.discount_widget.setLayout(FlowLayout())
        self.discount_stack.addWidget(WaitingSpinner())
        self.discount_stack.setCurrentIndex(1)

        self.game_widget.setLayout(FlowLayout())
        self.game_stack.addWidget(WaitingSpinner())
        self.game_stack.setCurrentIndex(1)

        self.search_bar = ButtonLineEdit(
            "fa.search", placeholder_text=self.tr("Search Games")
        )
        self.layout().insertWidget(0, self.search_bar)

        # self.search_bar.textChanged.connect(self.search_games)

        self.search_bar.returnPressed.connect(self.show_search_results)
        self.search_bar.buttonClicked.connect(self.show_search_results)

        self.init_filter()

        self.search_bar.setHidden(True)
        self.filter_gb.setHidden(True)
        self.filter_game_gb.setHidden(True)

        # self.search_bar.textChanged.connect(self.load_completer)

    def load(self):
        # load free games
        self.api_core.get_free_games(self.add_free_games)
        # load wishlist
        self.api_core.get_wishlist(self.add_wishlist_items)
        # load browse games
        self.prepare_request()

    def update_wishlist(self):
        self.api_core.get_wishlist(self.add_wishlist_items)

    def add_wishlist_items(self, wishlist):
        for i in range(self.discount_widget.layout().count()):
            item = self.discount_widget.layout().itemAt(i)
            if item:
                item.widget().deleteLater()

        if wishlist and wishlist[0] == "error":
            self.discount_widget.layout().addWidget(
                QLabel(self.tr("Failed to get wishlist: {}").format(wishlist[1]))
            )
            btn = QPushButton(self.tr("Reload"))
            self.discount_widget.layout().addWidget(btn)
            btn.clicked.connect(
                lambda: self.api_core.get_wishlist(self.add_wishlist_items)
            )
            self.discount_stack.setCurrentIndex(0)
            return

        discounts = 0
        for game in wishlist:
            if not game:
                continue
            try:
                if game["offer"]["price"]["totalPrice"]["discount"] > 0:
                    w = GameWidget(self.path, game["offer"])
                    w.show_info.connect(self.show_game.emit)
                    self.discount_widget.layout().addWidget(w)
                    discounts += 1
            except Exception as e:
                logger.warning(f"{game} {e}")
                continue
        self.discounts_gb.setVisible(discounts > 0)
        self.discount_stack.setCurrentIndex(0)
        # fix widget overlay
        self.discount_widget.layout().update()

    def add_free_games(self, free_games: list):
        for i in range(self.free_widget.layout().count()):
            item = self.free_widget.layout().itemAt(i)
            if item:
                item.widget().deleteLater()

        if free_games and free_games[0] == "error":
            self.free_widget.layout().addWidget(
                QLabel(self.tr("Failed to fetch free games: {}").format(free_games[1]))
            )
            btn = QPushButton(self.tr("Reload"))
            self.free_widget.layout().addWidget(btn)
            btn.clicked.connect(
                lambda: self.api_core.get_free_games(self.add_free_games)
            )
            self.free_stack.setCurrentIndex(0)
            return

        self.free_games_now = QGroupBox(self.tr("Now Free"))
        self.free_games_now.setLayout(QHBoxLayout())
        self.free_widget.layout().addWidget(self.free_games_now)

        self.coming_free_games = QGroupBox(self.tr("Free Games next week"))
        self.coming_free_games.setLayout(QHBoxLayout())
        self.free_widget.layout().addWidget(self.coming_free_games)

        date = datetime.datetime.now()
        free_games_now = []
        coming_free_games = []
        for game in free_games:
            try:
                if (
                    game["price"]["totalPrice"]["fmtPrice"]["discountPrice"] == "0"
                    and game["price"]["totalPrice"]["fmtPrice"]["originalPrice"]
                    != game["price"]["totalPrice"]["fmtPrice"]["discountPrice"]
                ):
                    free_games_now.append(game)
                    continue

                if game["title"] == "Mystery Game":
                    coming_free_games.append(game)
                    continue
            except KeyError as e:
                logger.warning(str(e))

            try:
                # parse datetime to check if game is next week or now
                try:
                    start_date = datetime.datetime.strptime(
                        game["promotions"]["upcomingPromotionalOffers"][0][
                            "promotionalOffers"
                        ][0]["startDate"],
                        "%Y-%m-%dT%H:%M:%S.%fZ",
                    )
                except Exception:
                    try:
                        start_date = datetime.datetime.strptime(
                            game["promotions"]["promotionalOffers"][0][
                                "promotionalOffers"
                            ][0]["startDate"],
                            "%Y-%m-%dT%H:%M:%S.%fZ",
                        )
                    except Exception as e:

                        continue

            except TypeError:
                print("type error")
                continue

            if start_date > date:
                coming_free_games.append(game)
        # free games now
        now_free = 0
        for free_game in free_games_now:
            w = GameWidget(self.path, free_game)
            w.show_info.connect(self.show_game.emit)
            self.free_games_now.layout().addWidget(w)
            self.free_game_widgets.append(w)
            now_free += 1
        if now_free == 0:
            self.free_games_now.layout().addWidget(
                QLabel(self.tr("Could not find current free game"))
            )

        # free games next week
        for free_game in coming_free_games:
            w = GameWidget(self.path, free_game)
            if free_game["title"] != "Mystery Game":
                w.show_info.connect(self.show_game.emit)
            self.coming_free_games.layout().addWidget(w)
        # self.coming_free_games.setFixedWidth(int(40 + len(coming_free_games) * 300))
        self.free_stack.setCurrentIndex(0)

    def show_search_results(self):
        if self.search_bar.text():
            self.show_info.emit(self.search_bar.text())

    def init_filter(self):
        self.none_price.toggled.connect(
            lambda: self.prepare_request("") if self.none_price.isChecked() else None
        )
        self.free_button.toggled.connect(
            lambda: self.prepare_request("free")
            if self.free_button.isChecked()
            else None
        )
        self.under10.toggled.connect(
            lambda: self.prepare_request("<price>[0, 1000)")
            if self.under10.isChecked()
            else None
        )
        self.under20.toggled.connect(
            lambda: self.prepare_request("<price>[0, 2000)")
            if self.under20.isChecked()
            else None
        )
        self.under30.toggled.connect(
            lambda: self.prepare_request("<price>[0, 3000)")
            if self.under30.isChecked()
            else None
        )
        self.above.toggled.connect(
            lambda: self.prepare_request("<price>[1499,]")
            if self.above.isChecked()
            else None
        )
        # self.on_discount.toggled.connect(lambda: self.prepare_request("sale") if self.on_discount.isChecked() else None)
        self.on_discount.toggled.connect(lambda: self.prepare_request())
        constants = Constants()

        self.checkboxes = []

        for groupbox, variables in [
            (self.genre_gb, constants.categories),
            (self.platform_gb, constants.platforms),
            (self.others_gb, constants.others),
            (self.type_gb, constants.types),
        ]:

            for text, tag in variables:
                checkbox = CheckBox(text, tag)
                checkbox.activated.connect(lambda x: self.prepare_request(added_tag=x))
                checkbox.deactivated.connect(
                    lambda x: self.prepare_request(removed_tag=x)
                )
                groupbox.layout().addWidget(checkbox)
                self.checkboxes.append(checkbox)
        self.reset_button.clicked.connect(self.reset_filters)

    def reset_filters(self):
        self.update_games_allowed = False
        for cb in self.checkboxes:
            cb.setChecked(False)
        self.none_price.setChecked(True)

        self.tags = []
        self.types = []
        self.update_games_allowed = True
        self.prepare_request("")

        self.on_discount.setChecked(False)

    def prepare_request(
        self,
        price: str = None,
        added_tag: int = 0,
        removed_tag: int = 0,
        added_type: str = "",
        removed_type: str = "",
    ):
        if not self.update_games_allowed:
            return
        if price is not None:
            self.price = price

        if added_tag != 0:
            self.tags.append(added_tag)
        if removed_tag != 0 and removed_tag in self.tags:
            self.tags.remove(removed_tag)

        if added_type:
            self.types.append(added_type)
        if removed_type and removed_type in self.types:
            self.types.remove(removed_type)
        if (self.types or self.price) or self.tags or self.on_discount.isChecked():
            self.free_game_group_box.setVisible(False)
            self.discounts_gb.setVisible(False)
        else:
            self.free_game_group_box.setVisible(True)
            if len(self.discounts_gb.layout().children()) > 0:
                self.discounts_gb.setVisible(True)

        self.game_stack.setCurrentIndex(1)

        browse_model = BrowseModel(
            language_code=self.core.language_code,
            country_code=self.core.country_code,
            count=20,
            price=self.price,
            onSale=self.on_discount.isChecked(),
        )
        browse_model.tag = "|".join(self.tags)

        if self.types:
            browse_model.category = "|".join(self.types)
        self.api_core.browse_games(browse_model, self.show_games)

    def show_games(self, data):
        for item in (
            self.game_widget.layout().itemAt(i)
            for i in range(self.game_widget.layout().count())
        ):
            item.widget().deleteLater()
        if data:
            for game in data:
                w = GameWidget(self.path, game, 275)
                self.game_widget.layout().addWidget(w)
                w.show_info.connect(self.show_game.emit)

        else:
            self.game_widget.layout().addWidget(
                QLabel(self.tr("Could not get games matching the filter"))
            )
        self.game_stack.setCurrentIndex(0)

        self.game_widget.layout().update()


class CheckBox(QCheckBox):
    activated = pyqtSignal(str)
    deactivated = pyqtSignal(str)

    def __init__(self, text, tag):
        super(CheckBox, self).__init__(text)
        self.tag = tag

        self.toggled.connect(self.handle_toggle)

    def handle_toggle(self):
        if self.isChecked():
            self.activated.emit(self.tag)
        else:
            self.deactivated.emit(self.tag)



import os
import shutil
from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QFileDialog, QWidget

from rare.components.tabs.settings.widgets.dxvk import DxvkSettings
from rare.components.tabs.settings.widgets.mangohud import MangoHudSettings
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton
from rare.ui.components.tabs.settings.linux import Ui_LinuxSettings
from rare.widgets.indicator_edit import PathEdit, IndicatorReasonsCommon
from rare.utils import config_helper

logger = getLogger("LinuxSettings")


class LinuxSettings(QWidget):
    # str: option key
    environ_changed = pyqtSignal(str)

    def __init__(self, name=None, parent=None):
        super(LinuxSettings, self).__init__(parent=parent)
        self.ui = Ui_LinuxSettings()
        self.ui.setupUi(self)

        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()

        self.name = name if name is not None else "default"

        # Wine prefix
        self.wine_prefix = PathEdit(
            self.load_prefix(),
            file_mode=QFileDialog.DirectoryOnly,
            edit_func=lambda path: (os.path.isdir(path) or not path, path, IndicatorReasonsCommon.DIR_NOT_EXISTS),
            save_func=self.save_prefix,
        )
        self.ui.prefix_layout.addWidget(self.wine_prefix)

        # Wine executable
        self.wine_exec = PathEdit(
            self.load_setting(self.name, "wine_executable"),
            file_mode=QFileDialog.ExistingFile,
            name_filters=["wine", "wine64"],
            edit_func=lambda text: (os.path.exists(text) or not text, text, IndicatorReasonsCommon.DIR_NOT_EXISTS),
            save_func=lambda text: self.save_setting(
                text, section=self.name, setting="wine_executable"
            ),
        )
        self.ui.exec_layout.addWidget(self.wine_exec)

        # dxvk
        self.dxvk = DxvkSettings()
        self.dxvk.environ_changed.connect(self.environ_changed)
        self.ui.linux_layout.addWidget(self.dxvk)
        self.dxvk.load_settings(self.name)

        self.mangohud = MangoHudSettings()
        self.mangohud.environ_changed.connect(self.environ_changed)
        self.ui.linux_layout.addWidget(self.mangohud)
        self.mangohud.load_settings(self.name)


    def load_prefix(self) -> str:
        return self.load_setting(
            f"{self.name}.env",
            "WINEPREFIX",
            fallback=self.load_setting(self.name, "wine_prefix"),
        )

    def save_prefix(self, text: str):
        self.save_setting(text, f"{self.name}.env", "WINEPREFIX")
        self.environ_changed.emit("WINEPREFIX")
        self.save_setting(text, self.name, "wine_prefix")
        self.signals.application.prefix_updated.emit()

    def load_setting(self, section: str, setting: str, fallback: str = ""):
        return self.core.lgd.config.get(section, setting, fallback=fallback)

    def save_setting(self, text: str, section: str, setting: str):
        if text:
            config_helper.add_option(section, setting, text)
            logger.debug(f"Set {setting} in {f'[{section}]'} to {text}")
        else:
            config_helper.remove_option(section, setting)
            logger.debug(f"Unset {setting} from {f'[{section}]'}")
        config_helper.save_config()



from typing import Optional

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QVBoxLayout, QWidget, QLabel, QSpacerItem, QSizePolicy

from rare.widgets.side_tab import SideTabWidget
from .egl_sync_group import EGLSyncGroup
from .eos_group import EOSGroup
from .import_group import ImportGroup
from .ubisoft_group import UbisoftGroup


class IntegrationsTabs(SideTabWidget):
    def __init__(self, parent=None):
        super(IntegrationsTabs, self).__init__(show_back=True, parent=parent)
        self.import_group = ImportGroup(self)
        self.import_widget = IntegrationsWidget(
            self.import_group,
            self.tr("To import games from Epic Games Store, please enable EGL Sync."),
            self,
        )
        self.import_index = self.addTab(self.import_widget, self.tr("Import Games"))

        self.egl_sync_group = EGLSyncGroup(self)
        self.egl_sync_widget = IntegrationsWidget(
            self.egl_sync_group,
            self.tr("To import EGL games from directories, please use Import Game."),
            self,
        )
        self.egl_sync_index = self.addTab(self.egl_sync_widget, self.tr("Sync with EGL"))

        self.eos_ubisoft = IntegrationsWidget(
            None,
            self.tr(""),
            self,
        )
        self.ubisoft_group = UbisoftGroup(self.eos_ubisoft)
        self.eos_group = EOSGroup(self.eos_ubisoft)
        self.eos_ubisoft.addWidget(self.eos_group)
        self.eos_ubisoft.addWidget(self.ubisoft_group)
        self.eos_ubisoft_index = self.addTab(self.eos_ubisoft, self.tr("Epic Overlay and Ubisoft"))

        self.setCurrentIndex(self.import_index)

    def show_import(self, app_name: str = None):
        self.setCurrentIndex(self.import_index)
        self.import_group.set_game(app_name)

    def show_egl_sync(self):
        self.setCurrentIndex(self.egl_sync_index)

    def show_eos_ubisoft(self):
        self.setCurrentIndex(self.eos_ubisoft_index)


class IntegrationsWidget(QWidget):
    def __init__(self, widget: Optional[QWidget], info: str, parent=None):
        super(IntegrationsWidget, self).__init__(parent=parent)
        self.info = QLabel(f"<b>{info}</b>")

        self.__layout = QVBoxLayout(self)
        if widget is not None:
            self.__layout.addWidget(widget)
        self.__layout.addWidget(self.info)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

    def addWidget(self, widget: QWidget, stretch: int = 0, alignment: Qt.AlignmentFlag = Qt.Alignment()):
        self.__layout.insertWidget(self.layout().count() - 1, widget, stretch, alignment)



import platform
from logging import getLogger

from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QDialog, QApplication
from requests.exceptions import ConnectionError, HTTPError

from rare.components.dialogs.login import LoginDialog
from rare.shared import RareCore
from rare.ui.components.dialogs.launch_dialog import Ui_LaunchDialog
from rare.widgets.elide_label import ElideLabel

logger = getLogger("LaunchDialog")


class LaunchDialog(QDialog):
    exit_app = pyqtSignal(int)
    start_app = pyqtSignal()

    def __init__(self, parent=None):
        super(LaunchDialog, self).__init__(parent=parent)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setWindowFlags(
            Qt.Window
            | Qt.Dialog
            | Qt.CustomizeWindowHint
            | Qt.WindowSystemMenuHint
            | Qt.WindowTitleHint
            | Qt.WindowMinimizeButtonHint
            | Qt.MSWindowsFixedSizeDialogHint
        )
        self.setWindowModality(Qt.WindowModal)
        self.ui = Ui_LaunchDialog()
        self.ui.setupUi(self)

        self.reject_close = True

        self.progress_info = ElideLabel(parent=self)
        self.progress_info.setFixedHeight(False)
        self.ui.launch_layout.addWidget(self.progress_info)

        self.rcore = RareCore.instance()
        self.rcore.progress.connect(self.__on_progress)
        self.rcore.completed.connect(self.__on_completed)
        self.core = self.rcore.core()
        self.args = self.rcore.args()

        self.login_dialog = LoginDialog(core=self.core, parent=parent)

    def login(self):
        do_launch = True
        try:
            if self.args.offline:
                pass
            else:
                # Force an update check and notice in case there are API changes
                self.core.check_for_updates(force=True)
                self.core.force_show_update = True
                if self.core.login():
                    logger.info("You are logged in")
                else:
                    raise ValueError("You are not logged in. Open Login Window")
        except ValueError as e:
            logger.info(str(e))
            # Do not set parent, because it won't show a task bar icon
            # Update: Inherit the same parent as LaunchDialog
            do_launch = self.login_dialog.login()
        except (HTTPError, ConnectionError) as e:
            logger.warning(e)
            self.args.offline = True
        finally:
            if do_launch:
                if not self.args.silent:
                    self.show()
                self.launch()
            else:
                self.exit_app.emit(0)

    def launch(self):
        self.progress_info.setText(self.tr("Preparing Rare"))
        self.rcore.fetch()

    @pyqtSlot(int, str)
    def __on_progress(self, i: int, m: str):
        self.ui.progress_bar.setValue(i)
        self.progress_info.setText(m)

    def __on_completed(self):
        logger.info("App starting")
        self.reject_close = False
        self.start_app.emit()

    def reject(self) -> None:
        if not self.reject_close:
            super(LaunchDialog, self).reject()



from abc import abstractmethod
from typing import Optional

from PyQt5.QtCore import QParallelAnimationGroup, Qt, QPropertyAnimation, QAbstractAnimation, QSize
from PyQt5.QtWidgets import (
    QWidget,
    QFrame,
    QToolButton,
    QVBoxLayout,
    QGridLayout,
    QSizePolicy,
    QGroupBox,
    QLabel,
)

from rare.utils.misc import icon


class CollapsibleBase(object):
    """
    References:
        # Adapted from c++ version
        https://stackoverflow.com/questions/32476006/how-to-make-an-expandable-collapsable-section-widget-in-qt
        # Adapted from python version
        https://newbedev.com/how-to-make-an-expandable-collapsable-section-widget-in-qt
    """
    def __init__(self):
        self.animation_duration = None
        self.toggle_animation = None
        self.content_area = None
        self.content_toggle_animation = None

    def setup(self, animation_duration: int = 200):
        self.animation_duration = animation_duration
        self.content_area: Optional[QWidget] = None
        self.content_toggle_animation: Optional[QPropertyAnimation] = None

        # let the entire widget grow and shrink with its content
        self.toggle_animation = QParallelAnimationGroup(self)
        self.toggle_animation.addAnimation(QPropertyAnimation(self, b"minimumHeight"))
        self.toggle_animation.addAnimation(QPropertyAnimation(self, b"maximumHeight"))

    @abstractmethod
    def isChecked(self) -> bool:
        pass

    @abstractmethod
    def click(self) -> None:
        pass

    @abstractmethod
    def addToLayout(self, widget: QWidget) -> None:
        pass

    @abstractmethod
    def sizeHint(self) -> QSize:
        pass

    def animationStart(self, checked):
        direction = QAbstractAnimation.Forward if checked else QAbstractAnimation.Backward
        self.toggle_animation.setDirection(direction)
        self.toggle_animation.start()

    def setWidget(self, widget: QWidget):
        if widget is None or widget is self.content_area:
            return
        is_checked = self.isChecked()
        if self.content_area is not None:
            # Collapse the parent before replacing the child
            if is_checked:
                self.click()
            self.toggle_animation.removeAnimation(self.content_toggle_animation)
            self.content_area.setParent(None)
            self.content_area.deleteLater()

        self.content_area = widget
        self.content_area.setParent(self)
        self.content_area.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

        # start out collapsed
        if not is_checked:
            self.content_area.setMaximumHeight(0)
            self.content_area.setMinimumHeight(0)

        self.content_toggle_animation = QPropertyAnimation(self.content_area, b"maximumHeight")
        self.toggle_animation.addAnimation(self.content_toggle_animation)
        self.addToLayout(self.content_area)
        collapsed_height = self.sizeHint().height()
        content_height = self.content_area.sizeHint().height()
        for i in range(self.toggle_animation.animationCount() - 1):
            spoiler_animation = self.toggle_animation.animationAt(i)
            spoiler_animation.setDuration(self.animation_duration)
            spoiler_animation.setStartValue(collapsed_height)
            spoiler_animation.setEndValue(collapsed_height + content_height)
        content_animation = self.toggle_animation.animationAt(self.toggle_animation.animationCount() - 1)
        content_animation.setDuration(self.animation_duration)
        content_animation.setStartValue(0)
        content_animation.setEndValue(content_height)
        if is_checked:
            self.click()


class CollapsibleFrame(QFrame, CollapsibleBase):
    def __init__(
        self, widget: QWidget = None, title: str = "", button_text: str = "", animation_duration: int = 200, parent=None
    ):
        super(CollapsibleFrame, self).__init__(parent=parent)
        self.setup(animation_duration)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Sunken)

        self.toggle_button = QToolButton(self)
        self.toggle_button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.toggle_button.setIcon(icon("fa.arrow-right"))
        self.toggle_button.setText(button_text)
        self.toggle_button.setCheckable(True)
        self.toggle_button.setChecked(False)

        self.title_label = QLabel(title)
        font = self.title_label.font()
        font.setBold(True)
        self.title_label.setFont(font)
        self.title_label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        # don't waste space
        self.main_layout = QGridLayout(self)
        self.main_layout.setVerticalSpacing(0)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.addWidget(self.toggle_button, 0, 0, 1, 1, Qt.AlignLeft)
        self.main_layout.addWidget(self.title_label, 0, 1, 1, 1)
        self.main_layout.setColumnStretch(1, 1)
        self.main_layout.setRowStretch(0, 0)
        self.main_layout.setRowStretch(1, 1)
        self.setLayout(self.main_layout)

        self.toggle_button.clicked.connect(self.animationStart)

        if widget is not None:
            self.setWidget(widget)

    def isChecked(self) -> bool:
        return self.toggle_button.isChecked()

    def click(self) -> None:
        self.toggle_button.click()

    def addToLayout(self, widget: QWidget) -> None:
        self.main_layout.addWidget(widget, 1, 0, 1, 2)

    def sizeHint(self) -> QSize:
        return super(CollapsibleFrame, self).sizeHint()

    def animationStart(self, checked):
        arrow_type = icon("fa.arrow-down") if checked else icon("fa.arrow-right")
        self.toggle_button.setIcon(arrow_type)
        super(CollapsibleFrame, self).animationStart(checked)


class CollapsibleGroupBox(QGroupBox, CollapsibleBase):
    def __init__(
        self, widget: QWidget = None, title: str = "", animation_duration: int = 200, parent=None
    ):
        super(CollapsibleGroupBox, self).__init__(parent=parent)
        self.setup(animation_duration)
        self.setTitle(title)
        self.setCheckable(True)
        self.setChecked(False)

        # don't waste space
        self.main_layout = QVBoxLayout(self)
        self.main_layout.setSpacing(0)
        self.main_layout.setContentsMargins(0, 0, 0, -1)
        self.setLayout(self.main_layout)

        self.toggled.connect(self.animationStart)

        if widget is not None:
            self.setWidget(widget)

    def isChecked(self) -> bool:
        return super(CollapsibleGroupBox, self).isChecked()

    def click(self) -> None:
        self.setChecked(not self.isChecked())

    def addToLayout(self, widget: QWidget) -> None:
        self.main_layout.addWidget(widget)

    def sizeHint(self) -> QSize:
        return super(CollapsibleGroupBox, self).sizeHint()


if __name__ == "__main__":
    import sys
    from PyQt5.QtWidgets import QApplication, QDialog
    from rare.ui.components.dialogs.install_dialog_advanced import Ui_InstallDialogAdvanced
    from rare.utils.misc import set_style_sheet
    from rare.resources.stylesheets import RareStyle
    app = QApplication(sys.argv)

    set_style_sheet("RareStyle")

    ui_frame = Ui_InstallDialogAdvanced()
    widget_frame = QWidget()
    ui_frame.setupUi(widget_frame)
    collapsible_frame = CollapsibleFrame(widget_frame, title="Frame me!")
    collapsible_frame.setDisabled(False)

    def replace_func_frame(state):
        widget2_frame = QWidget()
        ui2_frame = Ui_InstallDialogAdvanced()
        ui2_frame.setupUi(widget2_frame)
        if state:
            ui2_frame.install_dialog_advanced_layout.removeRow(3)
            ui2_frame.install_dialog_advanced_layout.removeRow(4)
        collapsible_frame.setWidget(widget2_frame)

    ui_group = Ui_InstallDialogAdvanced()
    widget_group = QWidget()
    ui_group.setupUi(widget_group)
    collapsible_group = CollapsibleGroupBox(widget_group, title="Group me!")
    collapsible_group.setDisabled(False)

    def replace_func_group(state):
        widget2_group = QWidget()
        ui2_group = Ui_InstallDialogAdvanced()
        ui2_group.setupUi(widget2_group)
        if state:
            ui2_group.install_dialog_advanced_layout.removeRow(3)
            ui2_group.install_dialog_advanced_layout.removeRow(4)
        collapsible_group.setWidget(widget2_group)

    replace_button = QToolButton()
    replace_button.setText("Replace me!")
    replace_button.setCheckable(True)
    replace_button.setChecked(False)
    replace_button.clicked.connect(replace_func_frame)
    replace_button.clicked.connect(replace_func_group)

    dialog = QDialog()
    dialog.setLayout(QVBoxLayout())
    dialog.layout().addWidget(replace_button)
    dialog.layout().addWidget(collapsible_frame)
    dialog.layout().addWidget(collapsible_group)
    dialog.layout().setSizeConstraint(QVBoxLayout.SetFixedSize)
    dialog.show()
    sys.exit(app.exec_())




import platform
import random
from logging import getLogger

from PyQt5.QtCore import pyqtSignal, Qt, pyqtSlot, QObject, QEvent, QTimer
from PyQt5.QtGui import QMouseEvent, QShowEvent
from PyQt5.QtWidgets import QMessageBox, QAction

from rare.models.game import RareGame
from rare.shared import (
    LegendaryCoreSingleton,
    GlobalSignalsSingleton,
    ArgumentsSingleton,
    ImageManagerSingleton,
)
from rare.utils.paths import desktop_links_supported, desktop_link_path, create_desktop_link
from .library_widget import LibraryWidget

logger = getLogger("GameWidget")


class GameWidget(LibraryWidget):
    show_info = pyqtSignal(RareGame)

    def __init__(self, rgame: RareGame, parent=None):
        super(GameWidget, self).__init__(parent=parent)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()
        self.image_manager = ImageManagerSingleton()

        self.rgame: RareGame = rgame

        self.setContextMenuPolicy(Qt.ActionsContextMenu)

        self.launch_action = QAction(self.tr("Launch"), self)
        self.launch_action.triggered.connect(self._launch)

        self.install_action = QAction(self.tr("Install"), self)
        self.install_action.triggered.connect(self._install)

        self.desktop_link_action = QAction(self)
        self.desktop_link_action.triggered.connect(
            lambda: self._create_link(self.rgame.folder_name, "desktop")
        )

        self.menu_link_action = QAction(self)
        self.menu_link_action.triggered.connect(
            lambda: self._create_link(self.rgame.folder_name, "start_menu")
        )

        self.reload_action = QAction(self.tr("Reload Image"), self)
        self.reload_action.triggered.connect(self._on_reload_image)

        self.uninstall_action = QAction(self.tr("Uninstall"), self)
        self.uninstall_action.triggered.connect(self._uninstall)

        self.update_actions()

        # signals
        self.rgame.signals.widget.update.connect(lambda: self.setPixmap(self.rgame.pixmap))
        self.rgame.signals.widget.update.connect(self.update_buttons)
        self.rgame.signals.widget.update.connect(self.update_state)
        self.rgame.signals.game.installed.connect(self.update_actions)
        self.rgame.signals.game.uninstalled.connect(self.update_actions)

        self.rgame.signals.progress.start.connect(
            lambda: self.showProgress(
                self.image_manager.get_pixmap(self.rgame.app_name, True),
                self.image_manager.get_pixmap(self.rgame.app_name, False)
            )
        )
        self.rgame.signals.progress.update.connect(
            lambda p: self.updateProgress(p)
        )
        self.rgame.signals.progress.finish.connect(
            lambda e: self.hideProgress(e)
        )

        self.state_strings = {
            RareGame.State.IDLE: "",
            RareGame.State.RUNNING: self.tr("Running..."),
            RareGame.State.DOWNLOADING: self.tr("Downloading..."),
            RareGame.State.VERIFYING: self.tr("Verifying..."),
            RareGame.State.MOVING: self.tr("Moving..."),
            RareGame.State.UNINSTALLING: self.tr("Uninstalling..."),
            RareGame.State.SYNCING: self.tr("Syncing saves..."),
            "has_update": self.tr("Update available"),
            "needs_verification": self.tr("Needs verification"),
            "not_can_launch": self.tr("Can't launch"),
            "save_not_up_to_date": self.tr("Save is not up-to-date")
        }

        self.hover_strings = {
            "info": self.tr("Show information"),
            "install": self.tr("Install game"),
            "can_launch": self.tr("Launch game"),
            "is_foreign": self.tr("Launch offline"),
            "has_update": self.tr("Launch without version check"),
            "is_origin": self.tr("Launch/Link"),
            "not_can_launch": self.tr("Can't launch"),
        }

    # lk: abstract class for typing, the `self.ui` attribute should be used
    # lk: by the Ui class in the children. It must contain at least the same
    # lk: attributes as `GameWidgetUi` class
    __slots__ = "ui"

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        if self.rgame.pixmap.isNull():
            QTimer.singleShot(random.randrange(42, 361, 7), self.rgame.load_pixmap)
        super().showEvent(a0)

    @pyqtSlot()
    def update_state(self):
        if self.rgame.is_idle:
            if self.rgame.has_update:
                self.ui.status_label.setText(self.state_strings["has_update"])
            elif self.rgame.needs_verification:
                self.ui.status_label.setText(self.state_strings["needs_verification"])
            elif not self.rgame.can_launch and self.rgame.is_installed:
                self.ui.status_label.setText(self.state_strings["not_can_launch"])
            elif self.rgame.igame and (
                    self.rgame.game.supports_cloud_saves or self.rgame.game.supports_mac_cloud_saves
            ) and not self.rgame.is_save_up_to_date:
                self.ui.status_label.setText(self.state_strings["save_not_up_to_date"])
            else:
                self.ui.status_label.setText(self.state_strings[self.rgame.state])
        else:
            self.ui.status_label.setText(self.state_strings[self.rgame.state])
        self.ui.status_label.setVisible(bool(self.ui.status_label.text()))

    @pyqtSlot()
    def update_buttons(self):
        self.ui.install_btn.setVisible(not self.rgame.is_installed)
        self.ui.install_btn.setEnabled(not self.rgame.is_installed)
        self.ui.launch_btn.setVisible(self.rgame.is_installed)
        self.ui.launch_btn.setEnabled(self.rgame.can_launch)

    @pyqtSlot()
    def update_actions(self):
        for action in self.actions():
            self.removeAction(action)

        if self.rgame.is_installed or self.rgame.is_origin:
            self.addAction(self.launch_action)
        else:
            self.addAction(self.install_action)

        if desktop_links_supported() and self.rgame.is_installed:
            if desktop_link_path(self.rgame.folder_name, "desktop").exists():
                self.desktop_link_action.setText(self.tr("Remove Desktop link"))
            else:
                self.desktop_link_action.setText(self.tr("Create Desktop link"))
            self.addAction(self.desktop_link_action)
            if desktop_link_path(self.rgame.folder_name, "start_menu").exists():
                self.menu_link_action.setText(self.tr("Remove Start Menu link"))
            else:
                self.menu_link_action.setText(self.tr("Create Start Menu link"))
            self.addAction(self.menu_link_action)

        self.addAction(self.reload_action)
        if self.rgame.is_installed and not self.rgame.is_origin:
            self.addAction(self.uninstall_action)

    def eventFilter(self, a0: QObject, a1: QEvent) -> bool:
        if a0 is self.ui.launch_btn:
            if a1.type() == QEvent.Enter:
                if not self.rgame.can_launch:
                    self.ui.tooltip_label.setText(self.hover_strings["not_can_launch"])
                elif self.rgame.is_origin:
                    self.ui.tooltip_label.setText(self.hover_strings["is_origin"])
                elif self.rgame.has_update:
                    self.ui.tooltip_label.setText(self.hover_strings["has_update"])
                elif self.rgame.is_foreign and self.rgame.can_run_offline:
                    self.ui.tooltip_label.setText(self.hover_strings["is_foreign"])
                elif self.rgame.can_launch:
                    self.ui.tooltip_label.setText(self.hover_strings["can_launch"])
                return True
            if a1.type() == QEvent.Leave:
                self.ui.tooltip_label.setText(self.hover_strings["info"])
                # return True
        if a0 is self.ui.install_btn:
            if a1.type() == QEvent.Enter:
                self.ui.tooltip_label.setText(self.hover_strings["install"])
                return True
            if a1.type() == QEvent.Leave:
                self.ui.tooltip_label.setText(self.hover_strings["info"])
                # return True
        if a0 is self:
            if a1.type() == QEvent.Enter:
                self.ui.tooltip_label.setText(self.hover_strings["info"])
        return super(GameWidget, self).eventFilter(a0, a1)

    def mousePressEvent(self, e: QMouseEvent) -> None:
        # left button
        if e.button() == 1:
            self.show_info.emit(self.rgame)
        # right
        elif e.button() == 2:
            super(GameWidget, self).mousePressEvent(e)

    @pyqtSlot()
    def _on_reload_image(self) -> None:
        self.rgame.refresh_pixmap()

    @pyqtSlot()
    @pyqtSlot(bool, bool)
    def _launch(self, offline=False, skip_version_check=False):
        if offline or (self.rgame.is_foreign and self.rgame.can_run_offline):
            offline = True
        if self.rgame.has_update:
            skip_version_check = True
        self.rgame.launch(
            offline=offline, skip_update_check=skip_version_check
        )

    @pyqtSlot()
    def _install(self):
        self.show_info.emit(self.rgame)

    @pyqtSlot()
    def _uninstall(self):
        self.show_info.emit(self.rgame)

    def _create_link(self, name, link_type):
        if not desktop_links_supported():
            QMessageBox.warning(
                self,
                self.tr("Warning"),
                self.tr("Creating shortcuts is currently unsupported on {}").format(platform.system()),
            )
            return

        shortcut_path = desktop_link_path(name, link_type)

        if not shortcut_path.exists():
            try:
                if not create_desktop_link(
                    app_name=self.rgame.app_name,
                    app_title=self.rgame.app_title,
                    link_name=self.rgame.folder_name,
                    link_type=link_type,
                ):
                    raise PermissionError
            except PermissionError:
                QMessageBox.warning(self, "Error", "Could not create shortcut.")
                return

            if link_type == "desktop":
                self.desktop_link_action.setText(self.tr("Remove Desktop link"))
            elif link_type == "start_menu":
                self.menu_link_action.setText(self.tr("Remove Start Menu link"))
        else:
            if shortcut_path.exists():
                shortcut_path.unlink(missing_ok=True)

            if link_type == "desktop":
                self.desktop_link_action.setText(self.tr("Create Desktop link"))
            elif link_type == "start_menu":
                self.menu_link_action.setText(self.tr("Create Start Menu link"))



"""
Module that overloads and monkeypatches legendary's classes/methods to work with Rare

Files with the 'api_' prefix are not part of legendary's source, and contain facilities relating to Rare.
"""


import os
import shutil
from enum import auto
from logging import getLogger
from typing import Tuple, Optional

from PyQt5.QtCore import pyqtSignal, Qt, pyqtSlot
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, QFileDialog

from rare.models.game import RareGame
from rare.shared import RareCore
from rare.utils.misc import path_size, format_size
from rare.widgets.elide_label import ElideLabel
from rare.widgets.indicator_edit import PathEdit, IndicatorReasons, IndicatorReasonsCommon

logger = getLogger("MoveGame")


class MovePathEditReasons(IndicatorReasons):
    DST_MISSING = auto()
    NO_WRITE_PERM = auto()
    SAME_DIR = auto()
    DST_IN_SRC = auto()
    NESTED_DIR = auto()
    NO_SPACE = auto()


class MoveGamePopUp(QWidget):
    move_clicked = pyqtSignal(str)
    browse_done = pyqtSignal()

    def __init__(self, parent=None):
        super(MoveGamePopUp, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.core = RareCore.instance().core()
        self.rgame: Optional[RareGame] = None

        self.path_edit = PathEdit("", QFileDialog.Directory, edit_func=self.path_edit_callback)
        self.path_edit.extend_reasons({
            MovePathEditReasons.DST_MISSING: self.tr("You need to provide the destination directory."),
            MovePathEditReasons.NO_WRITE_PERM: self.tr("No write permission on destination."),
            MovePathEditReasons.SAME_DIR: self.tr("Same directory or subdirectory selected."),
            MovePathEditReasons.DST_IN_SRC: self.tr("Destination is inside source directory"),
            MovePathEditReasons.NESTED_DIR: self.tr("Game install directories cannot be nested."),
            MovePathEditReasons.NO_SPACE: self.tr("Not enough space available on drive."),
        })
        self.path_edit.path_select.clicked.connect(self.browse_done)

        self.button = QPushButton(self.tr("Move"))
        self.button.setFixedSize(self.path_edit.path_select.sizeHint())
        self.button.clicked.connect(lambda p: self.move_clicked.emit(self.path_edit.text()))

        self.warn_label = ElideLabel("", parent=self)

        middle_layout = QHBoxLayout()
        middle_layout.setAlignment(Qt.AlignRight)
        middle_layout.addWidget(self.warn_label, stretch=1)
        middle_layout.addWidget(self.button)

        font = self.font()
        font.setBold(True)
        self.req_space_label = QLabel(self.tr("Required:"), self)
        self.req_space = QLabel(self)
        self.req_space.setFont(font)
        self.avail_space_label = QLabel(self.tr("Available:"), self)
        self.avail_space = QLabel(self)
        self.avail_space.setFont(font)

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.req_space_label)
        bottom_layout.addWidget(self.req_space, stretch=1)
        bottom_layout.addWidget(self.avail_space_label)
        bottom_layout.addWidget(self.avail_space, stretch=1)

        layout: QVBoxLayout = QVBoxLayout(self)
        layout.addWidget(self.path_edit)
        layout.addLayout(middle_layout)
        layout.addLayout(bottom_layout)

    def refresh_indicator(self):
        # needed so the edit_func gets run again
        text = self.path_edit.text()
        self.path_edit.setText(str())
        self.path_edit.setText(text)

    def path_edit_callback(self, path: str) -> Tuple[bool, str, int]:
        self.button.setEnabled(True)
        self.warn_label.setHidden(False)
        self.req_space.setText("...")
        self.avail_space.setText("...")

        def helper_func(reason: int) -> Tuple[bool, str, int]:
            self.button.setEnabled(False)
            return False, path, reason

        if not self.rgame.install_path or not path:
            return helper_func(MovePathEditReasons.DST_MISSING)

        src_path = os.path.realpath(self.rgame.install_path)
        dst_path = os.path.realpath(path)
        dst_install_path = os.path.realpath(os.path.join(dst_path, os.path.basename(src_path)))

        if not os.path.isdir(dst_path):
            return helper_func(IndicatorReasonsCommon.DIR_NOT_EXISTS)

        # Get free space on drive and size of game folder
        _, _, free_space = shutil.disk_usage(dst_path)
        source_size = path_size(src_path)

        # Calculate from bytes to gigabytes
        self.req_space.setText(format_size(source_size))
        self.avail_space.setText(format_size(free_space))

        if not os.access(path, os.W_OK) or not os.access(self.rgame.install_path, os.W_OK):
            return helper_func(MovePathEditReasons.NO_WRITE_PERM)

        if src_path == dst_path or src_path == dst_install_path:
            return helper_func(MovePathEditReasons.SAME_DIR)

        if str(src_path) in str(dst_path):
            return helper_func(MovePathEditReasons.DST_IN_SRC)

        if str(dst_install_path) in str(src_path):
            return helper_func(MovePathEditReasons.DST_IN_SRC)

        for rgame in self.rcore.installed_games:
            if not rgame.is_non_asset and rgame.install_path in path:
                return helper_func(MovePathEditReasons.NESTED_DIR)

        is_existing_dir = is_game_dir(src_path, dst_install_path)

        for item in os.listdir(dst_path):
            if os.path.basename(src_path) in os.path.basename(item):
                if os.path.isdir(dst_install_path):
                    if not is_existing_dir:
                        self.warn_label.setHidden(False)
                elif os.path.isfile(dst_install_path):
                    self.warn_label.setHidden(False)

        if free_space <= source_size and not is_existing_dir:
            return helper_func(MovePathEditReasons.NO_SPACE)

        # Fallback
        self.button.setEnabled(True)
        return True, path, IndicatorReasonsCommon.VALID

    @pyqtSlot()
    def __update_widget(self):
        """ React to state updates from RareGame """
        if not self.rgame.is_installed or self.rgame.is_non_asset:
            self.setDisabled(True)
            return
        # FIXME: Make edit_func lighter instead of blocking signals
        self.path_edit.line_edit.blockSignals(True)
        self.path_edit.setText(self.rgame.install_path)
        # FIXME: Make edit_func lighter instead of blocking signals
        self.path_edit.line_edit.blockSignals(False)
        self.warn_label.setText(
            self.tr("Moving here will overwrite <b>{}</b>").format(os.path.basename(self.rgame.install_path))
        )
        self.refresh_indicator()

    def update_game(self, rgame: RareGame):
        if self.rgame is not None:
            self.rgame.signals.widget.update.disconnect(self.__update_widget)
        self.rgame = None
        rgame.signals.widget.update.connect(self.__update_widget)
        self.rgame = rgame
        self.__update_widget()


def is_game_dir(src_path: str, dst_path: str):
    # This iterates over the destination dir, then iterates over the current install dir and if the file names
    # matches, we have an exisiting dir
    if os.path.isdir(dst_path):
        for dst_file in os.listdir(dst_path):
            for src_file in os.listdir(src_path):
                if dst_file == src_file:
                    return True
    return False



import platform as pf

from PyQt5.QtCore import QSettings, pyqtSignal, pyqtSlot, Qt
from PyQt5.QtWidgets import (
    QLabel,
    QPushButton,
    QWidget,
    QHBoxLayout,
    QComboBox, QToolButton, QMenu, QAction,
)
from qtawesome import IconWidget

from rare.shared import RareCore
from rare.utils.extra_widgets import SelectViewWidget, ButtonLineEdit
from rare.utils.misc import icon


class GameListHeadBar(QWidget):
    filterChanged: pyqtSignal = pyqtSignal(str)
    goto_import: pyqtSignal = pyqtSignal()
    goto_egl_sync: pyqtSignal = pyqtSignal()
    goto_eos_ubisoft: pyqtSignal = pyqtSignal()

    def __init__(self, parent=None):
        super(GameListHeadBar, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.settings = QSettings(self)

        self.filter = QComboBox(self)
        self.filter.addItem(self.tr("All games"), "all")
        self.filter.addItem(self.tr("Installed"), "installed")
        self.filter.addItem(self.tr("Offline"), "offline")
        # self.filter.addItem(self.tr("Hidden"), "hidden")
        if self.rcore.bit32_games:
            self.filter.addItem(self.tr("32bit games"), "32bit")
        if self.rcore.mac_games:
            self.filter.addItem(self.tr("macOS games"), "mac")
        if self.rcore.origin_games:
            self.filter.addItem(self.tr("Exclude Origin"), "installable")
        self.filter.addItem(self.tr("Include Unreal"), "include_ue")

        filter_default = "mac" if pf.system() == "Darwin" else "all"
        filter_index = i if (i := self.filter.findData(filter_default, Qt.UserRole)) >= 0 else 0
        try:
            self.filter.setCurrentIndex(self.settings.value("library_filter", filter_index, int))
        except TypeError:
            self.settings.setValue("library_filter", filter_index)
            self.filter.setCurrentIndex(filter_index)
        self.filter.currentIndexChanged.connect(self.filter_changed)

        integrations_menu = QMenu(self)
        import_action = QAction(icon("mdi.import", "fa.arrow-down"), self.tr("Import Game"), integrations_menu)

        import_action.triggered.connect(self.goto_import)
        egl_sync_action = QAction(icon("mdi.sync", "fa.refresh"), self.tr("Sync with EGL"), integrations_menu)
        egl_sync_action.triggered.connect(self.goto_egl_sync)

        eos_ubisoft_action = QAction(icon("mdi.rocket", "fa.rocket"), self.tr("Epic Overlay and Ubisoft"),
                                     integrations_menu)
        eos_ubisoft_action.triggered.connect(self.goto_eos_ubisoft)

        integrations_menu.addAction(import_action)
        integrations_menu.addAction(egl_sync_action)
        integrations_menu.addAction(eos_ubisoft_action)

        integrations = QToolButton(self)
        integrations.setText(self.tr("Integrations"))
        integrations.setMenu(integrations_menu)
        integrations.setPopupMode(QToolButton.InstantPopup)

        self.search_bar = ButtonLineEdit("fa.search", placeholder_text=self.tr("Search Game"))
        self.search_bar.setObjectName("SearchBar")
        self.search_bar.setFrame(False)
        self.search_bar.setMinimumWidth(200)

        checked = QSettings().value("icon_view", True, bool)

        installed_tooltip = self.tr("Installed games")
        self.installed_icon = IconWidget(parent=self)
        self.installed_icon.setIcon(icon("ph.floppy-disk-back-fill"))
        self.installed_icon.setToolTip(installed_tooltip)
        self.installed_label = QLabel(parent=self)
        font = self.installed_label.font()
        font.setBold(True)
        self.installed_label.setFont(font)
        self.installed_label.setToolTip(installed_tooltip)
        available_tooltip = self.tr("Available games")
        self.available_icon = IconWidget(parent=self)
        self.available_icon.setIcon(icon("ph.floppy-disk-back-light"))
        self.available_icon.setToolTip(available_tooltip)
        self.available_label = QLabel(parent=self)
        self.available_label.setToolTip(available_tooltip)

        self.view = SelectViewWidget(checked)

        self.refresh_list = QPushButton()
        self.refresh_list.setIcon(icon("fa.refresh"))  # Reload icon
        self.refresh_list.clicked.connect(self.refresh_clicked)

        layout = QHBoxLayout()
        layout.setContentsMargins(0, 5, 0, 5)
        layout.addWidget(self.filter)
        layout.addStretch(0)
        layout.addWidget(integrations)
        layout.addStretch(5)
        layout.addWidget(self.search_bar)
        layout.addStretch(2)
        layout.addWidget(self.installed_icon)
        layout.addWidget(self.installed_label)
        layout.addWidget(self.available_icon)
        layout.addWidget(self.available_label)
        layout.addStretch(2)
        layout.addWidget(self.view)
        layout.addStretch(2)
        layout.addWidget(self.refresh_list)
        self.setLayout(layout)

    def set_games_count(self, inst: int, avail: int) -> None:
        self.installed_label.setText(str(inst))
        self.available_label.setText(str(avail))

    @pyqtSlot()
    def refresh_clicked(self):
        self.rcore.fetch()

    @pyqtSlot(int)
    def filter_changed(self, index: int):
        self.filterChanged.emit(self.filter.itemData(index, Qt.UserRole))
        self.settings.setValue("library_filter", index)



from PyQt5.QtCore import QEvent, QObject, Qt
from PyQt5.QtGui import QShowEvent, QMovie
from PyQt5.QtWidgets import QLabel


class LoadingWidget(QLabel):
    def __init__(self, autostart=False, parent=None):
        super(LoadingWidget, self).__init__(parent=parent)
        self.setObjectName(type(self).__name__)
        self.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.movie = QMovie(":/images/loader.webp", parent=self)
        # The animation's exact size is 94x94
        self.setFixedSize(96, 96)
        self.setMovie(self.movie)
        if self.parent() is not None:
            self.parent().installEventFilter(self)
        if autostart:
            self.movie.start()

    def __center_on_parent(self):
        rect = self.rect()
        rect.moveCenter(self.parent().contentsRect().center())
        self.setGeometry(rect)

    def event(self, e: QEvent) -> bool:
        if e.type() == QEvent.ParentAboutToChange:
            if self.parent() is not None:
                self.parent().removeEventFilter(self)
        if e.type() == QEvent.ParentChange:
            if self.parent() is not None:
                self.parent().installEventFilter(self)
        return super().event(e)

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.__center_on_parent()
        super().showEvent(a0)

    def eventFilter(self, a0: QObject, a1: QEvent) -> bool:
        if a0 is self.parent() and a1.type() == QEvent.Resize:
            self.__center_on_parent()
            return a0.event(a1)
        return False

    def start(self):
        self.setVisible(True)
        self.movie.start()

    def stop(self):
        self.setVisible(False)
        self.movie.stop()



from dataclasses import dataclass, field
from email.message import Message
from logging import getLogger
from typing import Callable, Dict, TypeVar, List, Tuple
from typing import Union

import orjson
from PyQt5.QtCore import QObject, pyqtSignal, QUrl, QUrlQuery, pyqtSlot
from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkRequest, QNetworkReply, QNetworkDiskCache

REQUEST_LIMIT = 8
USER_AGENT = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36"
RequestHandler = TypeVar("RequestHandler", bound=Callable[[Union[Dict, bytes]], None])


@dataclass
class RequestQueueItem:
    method: str = None
    url: QUrl = None
    payload: Dict = field(default_factory=dict)
    params: Dict = field(default_factory=dict)
    handlers: List[RequestHandler] = field(default_factory=list)

    def __eq__(self, other):
        return self.method == other.method and self.url == other.url


class QtRequests(QObject):
    data_ready = pyqtSignal(object)

    def __init__(self, cache: str = None, token: str = None, parent=None):
        super(QtRequests, self).__init__(parent=parent)
        self.log = getLogger(f"{type(self).__name__}_{type(parent).__name__}")
        self.manager = QNetworkAccessManager(self)
        self.manager.finished.connect(self.__on_finished)
        self.manager.finished.connect(self.__process_next)
        self.cache = None
        if cache is not None:
            self.log.debug("Using cache dir %s", cache)
            self.cache = QNetworkDiskCache(self)
            self.cache.setCacheDirectory(cache)
            self.manager.setCache(self.cache)
        if token is not None:
            self.log.debug("Manager is authorized")
        self.token = token

        self.__pending_requests = []
        self.__active_requests = {}

    @staticmethod
    def __prepare_query(url, params) -> QUrl:
        url = QUrl(url)
        query = QUrlQuery(url)
        for k, v in params.items():
            query.addQueryItem(str(k), str(v))
        url.setQuery(query)
        return url

    def __prepare_request(self, item: RequestQueueItem) -> QNetworkRequest:
        request = QNetworkRequest(item.url)
        request.setHeader(QNetworkRequest.ContentTypeHeader, "application/json;charset=UTF-8")
        request.setHeader(QNetworkRequest.UserAgentHeader, USER_AGENT)
        request.setAttribute(QNetworkRequest.RedirectPolicyAttribute, QNetworkRequest.NoLessSafeRedirectPolicy)
        if self.cache is not None:
            request.setAttribute(QNetworkRequest.CacheLoadControlAttribute, QNetworkRequest.PreferCache)
        if self.token is not None:
            request.setRawHeader(b"Authorization", self.token.encode())
        return request

    def __post(self, item: RequestQueueItem):
        request = self.__prepare_request(item)
        payload = orjson.dumps(item.payload)  # pylint: disable=maybe-no-member
        reply = self.manager.post(request, payload)
        reply.errorOccurred.connect(self.__on_error)
        self.__active_requests[reply] = item

    def post(self, url: str, handler: RequestHandler, payload: dict):
        item = RequestQueueItem(method="post", url=QUrl(url), payload=payload, handlers=[handler])
        if len(self.__active_requests) < REQUEST_LIMIT:
            self.__post(item)
        else:
            self.__pending_requests.append(item)

    def __get(self, item: RequestQueueItem):
        request = self.__prepare_request(item)
        reply = self.manager.get(request)
        reply.errorOccurred.connect(self.__on_error)
        self.__active_requests[reply] = item

    def get(self, url: str, handler: RequestHandler, payload: Dict = None, params: Dict = None):
        url = self.__prepare_query(url, params) if params is not None else QUrl(url)
        item = RequestQueueItem(method="get", url=url, payload=payload, handlers=[handler])
        if len(self.__active_requests) < REQUEST_LIMIT:
            self.__get(item)
        else:
            self.__pending_requests.append(item)

    def __on_error(self, error: QNetworkReply.NetworkError) -> None:
        self.log.error(error)

    @staticmethod
    def __parse_content_type(header) -> Tuple[str, str]:
        # lk: this looks weird but `cgi` is deprecated, PEP 594 suggests this way of parsing MIME
        m = Message()
        m['content-type'] = header
        return m.get_content_type(), m.get_content_charset()

    def __process_next(self):
        if self.__pending_requests:
            item = self.__pending_requests.pop(0)
            if item.method == "post":
                self.__post(item)
            elif item.method == "get":
                self.__get(item)
            else:
                raise NotImplementedError

    @pyqtSlot(QNetworkReply)
    def __on_finished(self, reply: QNetworkReply):
        item = self.__active_requests.pop(reply, None)
        if item is None:
            self.log.error("QNetworkReply: %s without associated item", reply.url().toString())
            reply.deleteLater()
            return
        if reply.error():
            self.log.error(reply.errorString())
        else:
            mimetype, charset = self.__parse_content_type(reply.header(QNetworkRequest.ContentTypeHeader))
            maintype, subtype = mimetype.split("/")
            bin_data = reply.readAll().data()
            if mimetype == "application/json":
                data = orjson.loads(bin_data)
            elif maintype == "image":
                data = bin_data
            else:
                data = None
            for handler in item.handlers:
                handler(data)
        reply.deleteLater()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/utils/pathedit.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_PathEdit(object):
    def setupUi(self, PathEdit):
        PathEdit.setObjectName("PathEdit")
        PathEdit.resize(385, 30)
        PathEdit.setWindowTitle("PathEdit")
        self.layout_pathedit = QtWidgets.QHBoxLayout(PathEdit)
        self.layout_pathedit.setContentsMargins(0, 0, 0, 0)
        self.layout_pathedit.setObjectName("layout_pathedit")
        self.text_edit = QtWidgets.QLineEdit(PathEdit)
        self.text_edit.setMinimumSize(QtCore.QSize(300, 0))
        self.text_edit.setObjectName("text_edit")
        self.layout_pathedit.addWidget(self.text_edit)
        self.path_select = QtWidgets.QToolButton(PathEdit)
        self.path_select.setObjectName("path_select")
        self.layout_pathedit.addWidget(self.path_select)

        self.retranslateUi(PathEdit)

    def retranslateUi(self, PathEdit):
        _translate = QtCore.QCoreApplication.translate
        self.text_edit.setPlaceholderText(_translate("PathEdit", "Default"))
        self.path_select.setText(_translate("PathEdit", "Browse..."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    PathEdit = QtWidgets.QWidget()
    ui = Ui_PathEdit()
    ui.setupUi(PathEdit)
    PathEdit.show()
    sys.exit(app.exec_())



import platform
import time
from logging import getLogger

import pypresence.exceptions
from PyQt5.QtCore import QObject, QSettings
from pypresence import Presence

from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton

client_id = "830732538225360908"
logger = getLogger("RPC")


class DiscordRPC(QObject):
    def __init__(self):
        super(DiscordRPC, self).__init__()
        self.RPC = None
        self.state = 1  # 0: game, 1: always active, 2: off
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()

        self.settings = QSettings()
        if self.settings.value("rpc_enable", 0, int) == 1:  # show always
            self.state = 2
            self.set_discord_rpc()

        self.signals.discord_rpc.set_title.connect(self.update_presence)
        self.signals.discord_rpc.apply_settings.connect(self.changed_settings)

    def update_presence(self, app_name):
        self.set_discord_rpc(app_name)

    def changed_settings(self, game_running: list = None):
        value = self.settings.value("rpc_enable", 0, int)
        if value == 2:
            self.remove_rpc()
            return
        if not game_running and value == 0:
            self.remove_rpc()
        elif not game_running:
            self.set_discord_rpc()
        else:
            self.set_discord_rpc(game_running[0])

    def remove_rpc(self):
        if self.settings.value("rpc_enable", 0, int) != 1:
            if not self.RPC:
                return
            try:
                self.RPC.close()
            except Exception:
                logger.warning("Already closed")
            del self.RPC
            self.RPC = None
            logger.info("Remove RPC")
            self.state = 2
        else:
            self.state = 1
            self.set_discord_rpc()

    def set_discord_rpc(self, app_name=None):
        if not self.RPC:
            try:
                self.RPC = Presence(
                    client_id
                )  # Rare app: https://discord.com/developers/applications
                self.RPC.connect()
            except ConnectionRefusedError as e:
                logger.warning(f"Discord is not active\n{e}")
                self.RPC = None
                return
            except FileNotFoundError as e:
                logger.warning(f"File not found error\n{e}")
                self.RPC = None
                return
            except pypresence.exceptions.InvalidPipe as e:
                logger.error(f"Is Discord running? \n{e}")
                self.RPC = None
                return
            except Exception as e:
                logger.error(str(e))
                self.RPC = None
                return
        self.update_rpc(app_name)

    def update_rpc(self, app_name=None):
        if self.settings.value("rpc_enable", 0, int) == 2 or (
                not app_name and self.settings.value("rpc_enable", 0, int) == 0
        ):
            self.remove_rpc()
            return
        title = None
        if not app_name:
            self.RPC.update(
                large_image="logo", details="https://github.com/RareDevs/Rare"
            )
            return
        if self.settings.value("rpc_name", True, bool):
            try:
                title = self.core.get_installed_game(app_name).title
            except AttributeError:
                logger.error(f"Could not get title of game: {app_name}")
                title = app_name
        start = None
        if self.settings.value("rpc_time", True, bool):
            start = str(time.time()).split(".")[0]
        os = None
        if self.settings.value("rpc_os", True, bool):
            os = f"via Rare on {platform.system()}"

        self.RPC.update(
            large_image="logo", details=title, large_text=title, state=os, start=start
        )
        self.state = 0



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/widgets/overlay.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_OverlaySettings(object):
    def setupUi(self, OverlaySettings):
        OverlaySettings.setObjectName("OverlaySettings")
        OverlaySettings.resize(555, 188)
        OverlaySettings.setWindowTitle("Overlay Settings")
        OverlaySettings.setTitle("")
        self.dxvk_layout = QtWidgets.QGridLayout(OverlaySettings)
        self.dxvk_layout.setObjectName("dxvk_layout")
        self.lbl_show_overlay = QtWidgets.QLabel(OverlaySettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_show_overlay.sizePolicy().hasHeightForWidth())
        self.lbl_show_overlay.setSizePolicy(sizePolicy)
        self.lbl_show_overlay.setObjectName("lbl_show_overlay")
        self.dxvk_layout.addWidget(self.lbl_show_overlay, 0, 0, 1, 1)
        self.show_overlay_combo = QtWidgets.QComboBox(OverlaySettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_overlay_combo.sizePolicy().hasHeightForWidth())
        self.show_overlay_combo.setSizePolicy(sizePolicy)
        self.show_overlay_combo.setObjectName("show_overlay_combo")
        self.show_overlay_combo.addItem("")
        self.show_overlay_combo.addItem("")
        self.show_overlay_combo.addItem("")
        self.dxvk_layout.addWidget(self.show_overlay_combo, 0, 1, 1, 2)
        self.gb_options = QtWidgets.QGroupBox(OverlaySettings)
        self.gb_options.setTitle("")
        self.gb_options.setObjectName("gb_options")
        self.gridLayout = QtWidgets.QGridLayout(self.gb_options)
        self.gridLayout.setObjectName("gridLayout")
        self.options_form = QtWidgets.QFormLayout()
        self.options_form.setObjectName("options_form")
        self.gridLayout.addLayout(self.options_form, 1, 0, 1, 1)
        self.options_grid = QtWidgets.QGridLayout()
        self.options_grid.setObjectName("options_grid")
        self.gridLayout.addLayout(self.options_grid, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 1, 2, 1)
        self.dxvk_layout.addWidget(self.gb_options, 2, 0, 1, 3)

        self.retranslateUi(OverlaySettings)

    def retranslateUi(self, OverlaySettings):
        _translate = QtCore.QCoreApplication.translate
        self.lbl_show_overlay.setText(_translate("OverlaySettings", "Show HUD"))
        self.show_overlay_combo.setItemText(0, _translate("OverlaySettings", "System Default"))
        self.show_overlay_combo.setItemText(1, _translate("OverlaySettings", "Hidden"))
        self.show_overlay_combo.setItemText(2, _translate("OverlaySettings", "Visible"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    OverlaySettings = QtWidgets.QGroupBox()
    ui = Ui_OverlaySettings()
    ui.setupUi(OverlaySettings)
    OverlaySettings.show()
    sys.exit(app.exec_())



import webbrowser

from PyQt5.QtCore import pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QLabel, QPushButton

from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton
from rare.utils.misc import icon, ExitCodes


class AccountWidget(QWidget):
    exit_app: pyqtSignal = pyqtSignal(int)
    logout: pyqtSignal = pyqtSignal()

    def __init__(self, parent):
        super(AccountWidget, self).__init__(parent=parent)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()

        username = self.core.lgd.userdata.get("displayName")
        if not username:
            username = "Offline"

        self.open_browser = QPushButton(icon("fa.external-link"), self.tr("Account settings"))
        self.open_browser.clicked.connect(
            lambda: webbrowser.open(
                "https://www.epicgames.com/account/personal?productName=epicgames"
            )
        )
        self.logout_button = QPushButton(self.tr("Logout"), parent=self)
        self.logout_button.clicked.connect(self.__on_logout)
        self.quit_button = QPushButton(self.tr("Quit"), parent=self)
        self.quit_button.clicked.connect(self.__on_quit)

        layout = QVBoxLayout(self)
        layout.addWidget(QLabel(self.tr("Account")))
        layout.addWidget(QLabel(self.tr("Logged in as <b>{}</b>").format(username)))
        layout.addWidget(self.open_browser)
        layout.addWidget(self.logout_button)
        layout.addWidget(self.quit_button)

    @pyqtSlot()
    def __on_quit(self):
        self.exit_app.emit(ExitCodes.EXIT)

    @pyqtSlot()
    def __on_logout(self):
        self.exit_app.emit(ExitCodes.LOGOUT)



import os

from filelock import FileLock
from legendary.lfs.lgndry import LGDLFS as LGDLFSReal


class LGDLFS(LGDLFSReal):
    def __init__(self, *args, **kwargs):
        super(LGDLFS, self).__init__(*args, **kwargs)
        self.log.info("Using Rare's LGDLFS monkey")
        # Rare: Default FileLock in Python 3.11 is thread-local, so replace it with a non-local verison
        self._installed_lock = FileLock(os.path.join(self.path, 'installed.json') + '.lock', thread_local=False)

    def unlock_installed(self):
        self._installed_lock.release(force=True)



import time
from contextlib import contextmanager
from logging import Logger


@contextmanager
def timelogger(logger: Logger, title: str):
    start = time.perf_counter()
    yield
    logger.debug("%s: %s seconds", title, time.perf_counter() - start)



import functools
import json
import logging
import os
from multiprocessing import Queue
from sys import platform as sys_platform
from uuid import uuid4

# On Windows the monkeypatching of `run_real` below doesn't work like on Linux
# This has the side effect of emitting the UIUpdate in DownloadThread complaining with a TypeError
# So import `legendary.core` and monkeypatch its imported DLManager
import legendary.core
from legendary.core import LegendaryCore as LegendaryCoreReal
from legendary.lfs.utils import delete_folder
from legendary.models.downloading import AnalysisResult
from legendary.models.egl import EGLManifest
from legendary.models.game import Game, InstalledGame
from legendary.models.manifest import ManifestMeta

from rare.lgndr.downloader.mp.manager import DLManager
from rare.lgndr.glue.exception import LgndrException, LgndrLogHandler
from rare.lgndr.lfs.lgndry import LGDLFS

legendary.core.DLManager = DLManager
legendary.core.LGDLFS = LGDLFS


# fmt: off
class LegendaryCore(LegendaryCoreReal):

    def __init__(self, *args, **kwargs):
        super(LegendaryCore, self).__init__(*args, **kwargs)
        self.log.info("Using Rare's LegendaryCore monkey")
        self.log.info("Using config in %s", self.lgd.path.replace(os.getlogin(), "<username>"))
        self.handler = LgndrLogHandler(logging.CRITICAL)
        self.log.addHandler(self.handler)

    @staticmethod
    def unlock_installed(func):
        @functools.wraps(func)
        def unlock(self, *args, **kwargs):
            self.log.debug("%s: Using unlock decorator", func.__name__)
            if not self.lgd.lock_installed():
                self.log.info("Data is locked, trying to forcefully release it")
                # self.lgd._installed_lock.release(force=True)
            try:
                ret = func(self, *args, **kwargs)
            except Exception as e:
                raise e
            finally:
                self.lgd.unlock_installed()
            return ret
        return unlock

    @property
    def default_platform(self) -> str:
        os_default = "Mac" if sys_platform == "darwin" else "Windows"
        usr_platform = self.lgd.config.get("Legendary", "default_platform", fallback=os_default)
        return usr_platform if usr_platform in ("Windows", "Win32", "Mac") else os_default

    def update_check_enabled(self):
        return False

    def update_notice_enabled(self):
        return False

    # skip_sync defaults to false but since Rare is persistent, skip by default
    # def get_installed_game(self, app_name, skip_sync=True) -> InstalledGame:
    #     return super(LegendaryCore, self).get_installed_game(app_name, skip_sync)

    def prepare_download(self, game: Game, base_game: Game = None, base_path: str = '',
                         status_q: Queue = None, max_shm: int = 0, max_workers: int = 0,
                         force: bool = False, disable_patching: bool = False,
                         game_folder: str = '', override_manifest: str = '',
                         override_old_manifest: str = '', override_base_url: str = '',
                         platform: str = 'Windows', file_prefix_filter: list = None,
                         file_exclude_filter: list = None, file_install_tag: list = None,
                         dl_optimizations: bool = False, dl_timeout: int = 10,
                         repair: bool = False, repair_use_latest: bool = False,
                         disable_delta: bool = False, override_delta_manifest: str = '',
                         egl_guid: str = '', preferred_cdn: str = None,
                         disable_https: bool = False, bind_ip: str = None) -> (DLManager, AnalysisResult, ManifestMeta):
        dlm, analysis, igame = super(LegendaryCore, self).prepare_download(
            game=game, base_game=base_game, base_path=base_path,
            status_q=status_q, max_shm=max_shm, max_workers=max_workers,
            force=force, disable_patching=disable_patching,
            game_folder=game_folder, override_manifest=override_manifest,
            override_old_manifest=override_old_manifest, override_base_url=override_base_url,
            platform=platform, file_prefix_filter=file_prefix_filter,
            file_exclude_filter=file_exclude_filter, file_install_tag=file_install_tag,
            dl_optimizations=dl_optimizations, dl_timeout=dl_timeout,
            repair=repair, repair_use_latest=repair_use_latest,
            disable_delta=disable_delta, override_delta_manifest=override_delta_manifest,
            egl_guid=egl_guid, preferred_cdn=preferred_cdn,
            disable_https=disable_https, bind_ip=bind_ip,
        )
        # lk: monkeypatch run_real (the method that emits the stats) into DLManager
        # pylint: disable=E1111
        dlm.run_real = DLManager.run_real.__get__(dlm, DLManager)
        # lk: set the queue for reporting statistics back the UI
        dlm.status_queue = Queue()
        # lk: set the queue to send control signals to the DLManager
        # lk: this doesn't exist in the original class, but it is monkeypatched in
        dlm.signals_queue = Queue()
        return dlm, analysis, igame

    def uninstall_game(self, installed_game: InstalledGame, delete_files=True, delete_root_directory=False):
        try:
            super(LegendaryCore, self).uninstall_game(installed_game, delete_files, delete_root_directory)
        except Exception as e:
            raise e
        finally:
            pass

    @unlock_installed.__func__
    def egl_import(self, app_name):
        try:
            super(LegendaryCore, self).egl_import(app_name)
        except LgndrException as ret:
            raise ret
        finally:
            pass

    def egstore_write(self, app_name):
        self.log.debug(f'Exporting ".egstore" for "{app_name}"')
        # load igame/game
        lgd_game = self.get_game(app_name)
        lgd_igame = self._get_installed_game(app_name)
        manifest_data, _ = self.get_installed_manifest(app_name)
        if not manifest_data:
            self.log.error(f'Game Manifest for "{app_name}" not found, cannot export!')
            return

        # create guid if it's not set already
        if not lgd_igame.egl_guid:
            lgd_igame.egl_guid = str(uuid4()).replace('-', '').upper()
            _ = self._install_game(lgd_igame)
        # convert to egl manifest
        egl_game = EGLManifest.from_lgd_game(lgd_game, lgd_igame)

        # make sure .egstore folder exists
        egstore_folder = os.path.join(lgd_igame.install_path, '.egstore')
        if not os.path.exists(egstore_folder):
            os.makedirs(egstore_folder)

        # copy manifest and create mancpn file in .egstore folder
        with open(os.path.join(egstore_folder, f'{egl_game.installation_guid}.manifest', ), 'wb') as mf:
            mf.write(manifest_data)

        mancpn = dict(FormatVersion=0, AppName=app_name,
                      CatalogItemId=lgd_game.catalog_item_id,
                      CatalogNamespace=lgd_game.namespace)
        with open(os.path.join(egstore_folder, f'{egl_game.installation_guid}.mancpn', ), 'w') as mcpnf:
            json.dump(mancpn, mcpnf, indent=4, sort_keys=True)

    def egstore_delete(self, igame: InstalledGame, delete_files=True):
        self.log.debug(f'Removing ".egstore" for "{igame.app_name}"')
        if delete_files:
            delete_folder(os.path.join(igame.install_path, '.egstore'))

    @unlock_installed.__func__
    def egl_export(self, app_name):
        try:
            super(LegendaryCore, self).egl_export(app_name)
        except LgndrException as ret:
            raise ret
        finally:
            pass

    @unlock_installed.__func__
    def egl_sync(self, app_name=''):
        try:
            super(LegendaryCore, self).egl_sync(app_name)
        except LgndrException as ret:
            raise ret
        finally:
            pass

    def prepare_overlay_install(self, path=None):
        dlm, analysis_result, igame = super(LegendaryCore, self).prepare_overlay_install(path)
        # lk: monkeypatch status_q (the queue for download stats)
        # pylint: disable=E1111
        dlm.run_real = DLManager.run_real.__get__(dlm, DLManager)
        # lk: set the queue for reporting statistics back the UI
        dlm.status_queue = Queue()
        # lk: set the queue to send control signals to the DLManager
        # lk: this doesn't exist in the original class, but it is monkeypatched in
        dlm.signals_queue = Queue()
        return dlm, analysis_result, igame

# fmt: on



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/store/store.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_ShopWidget(object):
    def setupUi(self, ShopWidget):
        ShopWidget.setObjectName("ShopWidget")
        ShopWidget.resize(850, 572)
        ShopWidget.setWindowTitle("Form")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(ShopWidget)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.scrollArea = QtWidgets.QScrollArea(ShopWidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 828, 550))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.widget = QtWidgets.QWidget(self.scrollAreaWidgetContents)
        self.widget.setObjectName("widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.free_game_group_box = QtWidgets.QGroupBox(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.free_game_group_box.sizePolicy().hasHeightForWidth())
        self.free_game_group_box.setSizePolicy(sizePolicy)
        self.free_game_group_box.setObjectName("free_game_group_box")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.free_game_group_box)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.free_stack = QtWidgets.QStackedWidget(self.free_game_group_box)
        self.free_stack.setObjectName("free_stack")
        self.free_widget = QtWidgets.QWidget()
        self.free_widget.setObjectName("free_widget")
        self.free_stack.addWidget(self.free_widget)
        self.verticalLayout_3.addWidget(self.free_stack)
        self.verticalLayout.addWidget(self.free_game_group_box)
        self.discounts_gb = QtWidgets.QGroupBox(self.widget)
        self.discounts_gb.setObjectName("discounts_gb")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.discounts_gb)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.discount_stack = QtWidgets.QStackedWidget(self.discounts_gb)
        self.discount_stack.setObjectName("discount_stack")
        self.discount_widget = QtWidgets.QWidget()
        self.discount_widget.setObjectName("discount_widget")
        self.discount_stack.addWidget(self.discount_widget)
        self.verticalLayout_6.addWidget(self.discount_stack)
        self.verticalLayout.addWidget(self.discounts_gb)
        self.filter_game_gb = QtWidgets.QGroupBox(self.widget)
        self.filter_game_gb.setObjectName("filter_game_gb")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.filter_game_gb)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.game_stack = QtWidgets.QStackedWidget(self.filter_game_gb)
        self.game_stack.setObjectName("game_stack")
        self.game_widget = QtWidgets.QWidget()
        self.game_widget.setObjectName("game_widget")
        self.game_stack.addWidget(self.game_widget)
        self.verticalLayout_4.addWidget(self.game_stack)
        self.verticalLayout.addWidget(self.filter_game_gb)
        self.horizontalLayout.addWidget(self.widget)
        self.filter_gb = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filter_gb.sizePolicy().hasHeightForWidth())
        self.filter_gb.setSizePolicy(sizePolicy)
        self.filter_gb.setMinimumSize(QtCore.QSize(150, 0))
        self.filter_gb.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.filter_gb.setObjectName("filter_gb")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.filter_gb)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.reset_button = QtWidgets.QPushButton(self.filter_gb)
        self.reset_button.setObjectName("reset_button")
        self.verticalLayout_2.addWidget(self.reset_button)
        self.price_gb = QtWidgets.QGroupBox(self.filter_gb)
        self.price_gb.setObjectName("price_gb")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.price_gb)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.none_price = QtWidgets.QRadioButton(self.price_gb)
        self.none_price.setChecked(True)
        self.none_price.setObjectName("none_price")
        self.verticalLayout_9.addWidget(self.none_price)
        self.free_button = QtWidgets.QRadioButton(self.price_gb)
        self.free_button.setObjectName("free_button")
        self.verticalLayout_9.addWidget(self.free_button)
        self.under10 = QtWidgets.QRadioButton(self.price_gb)
        self.under10.setObjectName("under10")
        self.verticalLayout_9.addWidget(self.under10)
        self.under20 = QtWidgets.QRadioButton(self.price_gb)
        self.under20.setObjectName("under20")
        self.verticalLayout_9.addWidget(self.under20)
        self.under30 = QtWidgets.QRadioButton(self.price_gb)
        self.under30.setObjectName("under30")
        self.verticalLayout_9.addWidget(self.under30)
        self.above = QtWidgets.QRadioButton(self.price_gb)
        self.above.setObjectName("above")
        self.verticalLayout_9.addWidget(self.above)
        self.on_discount = QtWidgets.QCheckBox(self.price_gb)
        self.on_discount.setObjectName("on_discount")
        self.verticalLayout_9.addWidget(self.on_discount)
        self.verticalLayout_2.addWidget(self.price_gb)
        self.platform_gb = QtWidgets.QGroupBox(self.filter_gb)
        self.platform_gb.setObjectName("platform_gb")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.platform_gb)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.verticalLayout_2.addWidget(self.platform_gb)
        self.genre_gb = QtWidgets.QGroupBox(self.filter_gb)
        self.genre_gb.setObjectName("genre_gb")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.genre_gb)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.verticalLayout_2.addWidget(self.genre_gb)
        self.type_gb = QtWidgets.QGroupBox(self.filter_gb)
        self.type_gb.setObjectName("type_gb")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.type_gb)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.verticalLayout_2.addWidget(self.type_gb)
        self.others_gb = QtWidgets.QGroupBox(self.filter_gb)
        self.others_gb.setObjectName("others_gb")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.others_gb)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_2.addWidget(self.others_gb)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout.addWidget(self.filter_gb)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.horizontalLayout_2.addWidget(self.scrollArea)
        self.verticalLayout_7.addLayout(self.horizontalLayout_2)

        self.retranslateUi(ShopWidget)

    def retranslateUi(self, ShopWidget):
        _translate = QtCore.QCoreApplication.translate
        self.free_game_group_box.setTitle(_translate("ShopWidget", "Free Games"))
        self.discounts_gb.setTitle(_translate("ShopWidget", "Discounts from your wishlist"))
        self.filter_game_gb.setTitle(_translate("ShopWidget", "Games"))
        self.filter_gb.setTitle(_translate("ShopWidget", "Filter"))
        self.reset_button.setText(_translate("ShopWidget", "Reset"))
        self.price_gb.setTitle(_translate("ShopWidget", "Price"))
        self.none_price.setText(_translate("ShopWidget", "None"))
        self.free_button.setText(_translate("ShopWidget", "Free"))
        self.under10.setText(_translate("ShopWidget", "Under 10"))
        self.under20.setText(_translate("ShopWidget", "Under 20"))
        self.under30.setText(_translate("ShopWidget", "Under 30"))
        self.above.setText(_translate("ShopWidget", "14.99 and above"))
        self.on_discount.setText(_translate("ShopWidget", "Discount"))
        self.platform_gb.setTitle(_translate("ShopWidget", "Platform"))
        self.genre_gb.setTitle(_translate("ShopWidget", "Genre"))
        self.type_gb.setTitle(_translate("ShopWidget", "Type"))
        self.others_gb.setTitle(_translate("ShopWidget", "Other Tags"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ShopWidget = QtWidgets.QWidget()
    ui = Ui_ShopWidget()
    ui.setupUi(ShopWidget)
    ShopWidget.show()
    sys.exit(app.exec_())
